@@ -59,10 +59,12 @@ if typing.TYPE_CHECKING:
     from gajim.gui.application import GajimApplication
     from gajim.common.storage.cache import CacheStorage
     from gajim.common.storage.archive import MessageArchiveStorage
+    from gajim.common.storage.events import EventStorage
     from gajim.common.cert_store import CertificateStore
     from gajim.common.call_manager import CallManager
     from gajim.common.preview import PreviewManager
     from gajim.common.task_manager import TaskManager
+    from gajim.common.commands import ChatCommands
 
 
 interface = cast(types.InterfaceT, None)
@@ -74,9 +76,9 @@ version = gajim.__version__
 connections: dict[str, types.Client] = {}
 avatar_cache: dict[str, dict[str, Any]] = {}
 bob_cache: dict[str, bytes] = {}
-ipython_window = None
 app = None  # type: GajimApplication
 window = None  # type: MainWindow
+commands = None  # type: ChatCommands
 
 ged = ged_module.GlobalEventsDispatcher()  # Global Events Dispatcher
 plugin_manager = cast(types.PluginManagerT, None)  # Plugins Manager
@@ -87,6 +89,7 @@ class Storage:
     def __init__(self):
         self.cache: CacheStorage = None
         self.archive: MessageArchiveStorage = None
+        self.events: EventStorage = None
 
 
 storage = Storage()
