@@ -36,6 +36,8 @@ from gajim.common.i18n import Q_
 STOP_EVENT = True
 PROPAGATE_EVENT = False
 
+MAX_MESSAGE_CORRECTION_DELAY = 300
+
 
 class EncryptionData(NamedTuple):
     additional_data: Any = None
@@ -49,9 +51,8 @@ class Entity(NamedTuple):
 
 
 class RowHeaderType(IntEnum):
-    ACTIVE = 0
-    CONVERSATIONS = 1
-    PINNED = 2
+    CONVERSATIONS = 0
+    PINNED = 1
 
 
 class AvatarSize(IntEnum):
@@ -111,16 +112,6 @@ class KindConstant(IntEnum):
         return str(self.value)
 
 
-@unique
-class ShowConstant(IntEnum):
-    ONLINE = 0
-    CHAT = 1
-    AWAY = 2
-    XA = 3
-    DND = 4
-    OFFLINE = 5
-
-
 @unique
 class TypeConstant(IntEnum):
     AIM = 0
@@ -163,6 +154,7 @@ class StyleAttr(Enum):
 
 @unique
 class CSSPriority(IntEnum):
+    PRE_APPLICATION = 599
     APPLICATION = 600
     APPLICATION_DARK = 601
     DEFAULT_THEME = 610
@@ -246,6 +238,7 @@ class MUCJoinedState(Enum):
     CREATING = 'creating'
     CAPTCHA_REQUEST = 'captcha in progress'
     CAPTCHA_FAILED = 'captcha failed'
+    PASSWORD_REQUEST = 'password request'
 
     def __str__(self):
         return self.name
@@ -274,6 +267,10 @@ class MUCJoinedState(Enum):
     def is_captcha_failed(self):
         return self == MUCJoinedState.CAPTCHA_FAILED
 
+    @property
+    def is_password_request(self):
+        return self == MUCJoinedState.PASSWORD_REQUEST
+
 
 class ClientState(IntEnum):
     DISCONNECTING = 0
@@ -1186,3 +1183,12 @@ class PresenceShowExt(Enum):
         if not isinstance(other, PresenceShow):  # pyright: ignore
             return NotImplemented
         return True
+
+
+# Replace keys with Gio.TlsProtocolVersion enum once
+# we require GLib >= 2.69
+TLS_VERSION_STRINGS = {
+    3: '1.1',
+    4: '1.2',
+    5: '1.3',
+}
