@@ -37,7 +37,6 @@ import sys
 import re
 import os
 import subprocess
-import base64
 import hashlib
 import shlex
 import socket
@@ -57,17 +56,20 @@ from string import Template
 from datetime import datetime
 from datetime import timedelta
 from urllib.parse import unquote
-from encodings.punycode import punycode_encode
+from urllib.parse import urlparse
 from functools import wraps
 from pathlib import Path
 from packaging.version import Version as V
+import unicodedata
 
 from nbxmpp.namespaces import Namespace
 from nbxmpp.const import Role
+from nbxmpp.const import Chatstate
 from nbxmpp.const import ConnectionProtocol
 from nbxmpp.const import ConnectionType
 from nbxmpp.const import Affiliation
 from nbxmpp.errors import StanzaError
+from nbxmpp.structs import CommonError
 from nbxmpp.structs import ProxyData
 from nbxmpp.protocol import JID
 from nbxmpp.protocol import InvalidJid
@@ -303,22 +305,23 @@ def from_one_line(msg: str) -> str:
     return msg
 
 
-def get_uf_chatstate(chatstate: str) -> str:
-    '''
-    Remove chatstate jargon and returns user friendly messages
-    '''
-    if chatstate == 'active':
-        return _('is paying attention to the conversation')
-    if chatstate == 'inactive':
-        return _('is doing something else')
-    if chatstate == 'composing':
+def chatstate_to_string(chatstate: Optional[Chatstate]) -> str:
+    if chatstate is None:
+        return ''
+
+    if chatstate == Chatstate.ACTIVE:
+        return ''
+
+    if chatstate == Chatstate.COMPOSING:
         return _('is composing a message…')
-    if chatstate == 'paused':
-        # paused means he or she was composing but has stopped for a while
+
+    if chatstate in (Chatstate.INACTIVE, Chatstate.GONE):
+        return _('is doing something else')
+
+    if chatstate == Chatstate.PAUSED:
         return _('paused composing a message')
-    if chatstate == 'gone':
-        return _('has closed the chat window or tab')
-    return ''
+
+    raise ValueError('unknown value: %s' % chatstate)
 
 
 def exec_command(command: str,
@@ -361,28 +364,40 @@ def get_file_path_from_dnd_dropped_uri(uri: str) -> str:
 
 def sanitize_filename(filename: str) -> str:
     '''
-    Make sure the filename we will write does contain only acceptable and latin
-    characters, and is not too long (in that case hash it)
+    Sanitize filename of elements not allowed on Windows
+    https://docs.microsoft.com/en-us/windows/win32/fileio/naming-a-file
+    Limit filename length to 50 chars on all systems
     '''
-    # 48 is the limit
-    if len(filename) > 48:
-        hash_ = hashlib.md5(filename.encode('utf-8'))
-        filename = base64.b64encode(hash_.digest()).decode('utf-8')
+    if sys.platform == 'win32':
+        blacklist = ['\\', '/', ':', '*', '?', '"', '<', '>', '|', '\0']
+        reserved_filenames = [
+            'CON', 'PRN', 'AUX', 'NUL', 'COM1', 'COM2', 'COM3', 'COM4', 'COM5',
+            'COM6', 'COM7', 'COM8', 'COM9', 'LPT1', 'LPT2', 'LPT3', 'LPT4',
+            'LPT5', 'LPT6', 'LPT7', 'LPT8', 'LPT9',
+        ]
+        filename = ''.join(char for char in filename if char not in blacklist)
 
-    # make it latin chars only
-    filename = punycode_encode(filename).decode('utf-8')
-    filename = filename.replace('/', '_')
-    if os.name == 'nt':
-        filename = filename.replace('?', '_')\
-                           .replace(':', '_')\
-                           .replace('\\', '_')\
-                           .replace('"', "'")\
-                           .replace('|', '_')\
-                           .replace('*', '_')\
-                           .replace('<', '_')\
-                           .replace('>', '_')
+        filename = ''.join(char for char in filename if 31 < ord(char))
 
-    return filename
+        filename = unicodedata.normalize('NFKD', filename)
+        filename = filename.rstrip('. ')
+        filename = filename.strip()
+
+        if all(char == '.' for char in filename):
+            filename = f'__{filename}'
+        if filename.upper() in reserved_filenames:
+            filename = f'__{filename}'
+        if len(filename) == 0:
+            filename = '__'
+
+    extension = Path(filename).suffix[:10]
+    filename = Path(filename).stem
+    final_length = 50 - len(extension)
+
+    # Many Filesystems have a limit on filename length: keep it short
+    filename = filename[:final_length]
+
+    return f'{filename}{extension}'
 
 
 def get_contact_dict_for_account(account: str) -> dict[str, types.BareContact]:
@@ -494,11 +509,11 @@ def get_client_status(account: str) -> str:
 
 def get_global_show() -> str:
     maxi = 0
-    for account in app.connections:
-        if not app.settings.get_account_setting(account,
+    for client in app.get_clients():
+        if not app.settings.get_account_setting(client.account,
                                                 'sync_with_global_status'):
             continue
-        status = get_client_status(account)
+        status = get_client_status(client.account)
         index = SHOW_LIST.index(status)
         if index > maxi:
             maxi = index
@@ -508,14 +523,14 @@ def get_global_show() -> str:
 def get_global_status_message() -> str:
     maxi = 0
     status_message = ''
-    for account, con in app.connections.items():
-        if not app.settings.get_account_setting(account,
+    for client in app.get_clients():
+        if not app.settings.get_account_setting(client.account,
                                                 'sync_with_global_status'):
             continue
-        index = SHOW_LIST.index(con.status)
+        index = SHOW_LIST.index(client.status)
         if index > maxi:
             maxi = index
-            status_message = con.status_message
+            status_message = client.status_message
     return status_message
 
 
@@ -524,7 +539,8 @@ def statuses_unified() -> bool:
     Test if all statuses are the same
     '''
     reference = None
-    for account in app.connections:
+    for client in app.get_clients():
+        account = client.account
         if not app.settings.get_account_setting(account,
                                                 'sync_with_global_status'):
             continue
@@ -655,9 +671,11 @@ def get_optional_features(account: str) -> list[str]:
 
     features.append(Namespace.NICK + '+notify')
 
-    if app.connections[account].get_module('Bookmarks').nativ_bookmarks_used:
+    client = app.get_client(account)
+
+    if client.get_module('Bookmarks').nativ_bookmarks_used:
         features.append(Namespace.BOOKMARKS_1 + '+notify')
-    elif app.connections[account].get_module('Bookmarks').pep_bookmarks_used:
+    elif client.get_module('Bookmarks').pep_bookmarks_used:
         features.append(Namespace.BOOKMARKS + '+notify')
     if app.is_installed('AV'):
         features.append(Namespace.JINGLE_RTP)
@@ -671,8 +689,8 @@ def get_optional_features(account: str) -> list[str]:
 
 
 def jid_is_blocked(account: str, jid: str) -> bool:
-    con = app.connections[account]
-    return jid in con.get_module('Blocking').blocked
+    client = app.get_client(account)
+    return jid in client.get_module('Blocking').blocked
 
 
 def get_subscription_request_msg(account: Optional[str] = None) -> str:
@@ -801,18 +819,6 @@ def load_json(path: Path,
     return json_dict.get(key, default)
 
 
-def ignore_contact(account: str, jid: JID) -> bool:
-    client = app.get_client(account)
-    contact = client.get_module('Contacts').get_contact(jid)
-
-    ignore_unknown = app.settings.get_account_setting(
-        account, 'ignore_unknown_contacts')
-    if ignore_unknown and not contact.is_in_roster:
-        log.info('Ignore unknown contact %s', str(jid))
-        return True
-    return False
-
-
 class AdditionalDataDict(collections.UserDict):
     data: dict[str, Any]
 
@@ -990,6 +996,13 @@ def parse_uri(uri: str) -> URI:
     if uri.startswith('file://'):
         return URI(type=URIType.FILE, data=uri)
 
+    try:
+        urlparts = urlparse(uri)
+        if not urlparts.scheme:
+            return URI(type=URIType.WEB, data=f'http://{uri}')
+    except Exception:
+        pass
+
     return URI(type=URIType.WEB, data=uri)
 
 
@@ -1145,7 +1158,7 @@ def validate_jid(jid: Union[str, JID], type_: Optional[str] = None) -> JID:
     raise ValueError(f'Not a {type_} JID')
 
 
-def to_user_string(error: StanzaError) -> str:
+def to_user_string(error: Union[CommonError, StanzaError]) -> str:
     text = error.get_text(get_rfc5646_lang())
     if text:
         return text
@@ -1220,19 +1233,46 @@ class Observable:
         self._log = log_
         self._callbacks: types.ObservableCbDict = defaultdict(list)
 
-    def disconnect_signals(self) -> None:
-        self._callbacks = defaultdict(list)
+    def __disconnect(self,
+                     obj: Any,
+                     signals: Optional[set[str]] = None
+                     ) -> None:
+
+        def _remove(handlers: list[weakref.WeakMethod[types.AnyCallableT]]
+                    ) -> None:
 
-    def disconnect_all_from_obj(self, obj: Any) -> None:
-        for handlers in self._callbacks.values():
             for handler in list(handlers):
                 func = handler()
                 if func is None or func.__self__ is obj:
                     handlers.remove(handler)
 
+        if signals is None:
+            for handlers in self._callbacks.values():
+                _remove(handlers)
+
+        else:
+            for signal in signals:
+                _remove(self._callbacks.get(signal, []))
+
+    def disconnect_signals(self) -> None:
+        self._callbacks = defaultdict(list)
+
+    def multi_disconnect(self,
+                         obj: Any,
+                         signals: Optional[set[str]]
+                         ) -> None:
+
+        self.__disconnect(obj, signals)
+
+    def disconnect_all_from_obj(self, obj: Any) -> None:
+        self.__disconnect(obj)
+
     def disconnect(self, obj: Any) -> None:
         self.disconnect_all_from_obj(obj)
 
+    def disconnect_signal(self, obj: Any, signal: str) -> None:
+        self.__disconnect(obj, {signal})
+
     def connect_signal(self,
                        signal_name: str,
                        func: types.AnyCallableT) -> None:
@@ -1387,16 +1427,6 @@ def get_group_chat_nick(account: str, room_jid: Union[JID, str]) -> str:
     return nick
 
 
-def get_muc_context(jid: JID) -> Optional[str]:
-    disco_info = app.storage.cache.get_last_disco_info(jid)
-    if disco_info is None:
-        return None
-
-    if (disco_info.muc_is_members_only and disco_info.muc_is_nonanonymous):
-        return 'private'
-    return 'public'
-
-
 def get_random_muc_localpart() -> str:
     rand = random.randrange(4)
     is_vowel = bool(random.getrandbits(1))
