@@ -32,7 +32,6 @@ from pathlib import Path
 from gi.repository import GLib
 
 DOMAIN = 'gajim'
-direction_mark = '\u200E'
 _translation = gettext.NullTranslations()
 
 
@@ -54,7 +53,7 @@ def get_locale_dirs() -> Optional[list[Path]]:
     return [Path(dir_) for dir_ in data_dirs]
 
 
-def iter_locale_dirs() -> Iterator[Optional[str]]:
+def iter_locale_dirs() -> Iterator[Optional[Path]]:
     locale_dirs = get_locale_dirs()
     if locale_dirs is None:
         yield None
@@ -70,7 +69,7 @@ def iter_locale_dirs() -> Iterator[Optional[str]]:
             continue
         found_paths.append(locale_dir)
         if locale_dir.is_dir():
-            yield str(locale_dir)
+            yield locale_dir
 
 
 def get_win32_default_lang() -> str:
@@ -108,30 +107,19 @@ def get_short_lang_code(lang: Optional[str] = None) -> str:
     return lang[:2]
 
 
-def initialize_direction_mark() -> None:
-    from gi.repository import Gtk
-
-    global direction_mark  # pylint: disable=global-statement
-
-    if Gtk.Widget.get_default_direction() == Gtk.TextDirection.RTL:
-        direction_mark = '\u200F'
-
-
-def paragraph_direction_mark(text: str) -> str:
+def is_rtl_text(text: str) -> bool:
     '''
     Determine paragraph writing direction according to
     http://www.unicode.org/reports/tr9/#The_Paragraph_Level
-
-    Returns either Unicode LTR mark or RTL mark.
     '''
     for char in text:
         bidi = unicodedata.bidirectional(char)
         if bidi == 'L':
-            return '\u200E'
+            return False
         if bidi in ('AL', 'R'):
-            return '\u200F'
+            return True
 
-    return '\u200E'
+    return False
 
 
 def Q_(text: str) -> str:
