@@ -59,6 +59,7 @@ class Caps(BaseModule):
         self.handlers = [
             StanzaHandler(name='presence',
                           callback=self._entity_caps,
+                          typ='available',
                           ns=Namespace.CAPS,
                           priority=51),
         ]
@@ -103,8 +104,6 @@ class Caps(BaseModule):
                      _stanza: Presence,
                      properties: PresenceProperties
                      ) -> None:
-        if properties.type.is_error or properties.type.is_unavailable:
-            return
 
         if properties.is_self_presence:
             return
@@ -112,6 +111,10 @@ class Caps(BaseModule):
         if properties.entity_caps is None:
             return
 
+        if properties.muc_user is not None:
+            # Don’t query MUC participants
+            return
+
         task = EntityCapsTask(self._account, properties, self._execute_task)
 
         self._log.info('Received %s', task.entity)
@@ -190,9 +193,9 @@ class Caps(BaseModule):
         if not app.account_is_available(self._account):
             return
 
-        app.connections[self._account].change_status(
-            app.connections[self._account].status,
-            app.connections[self._account].status_message)
+        client = app.get_client(self._account)
+        client.change_status(client.status,
+                             client.status_message)
 
     def cleanup(self) -> None:
         self._remove_all_tasks()
