@@ -80,6 +80,7 @@ class Preview:
         self.mime_type: str = ''
         self.file_size: int = 0
         self._received_size: int = 0
+        self.download_in_progress = False
 
         self.info_message: Optional[str] = None
 
@@ -143,11 +144,13 @@ class Preview:
             return False
         return self._urlparts.scheme == 'aesgcm'
 
+    @property
     def thumb_exists(self) -> bool:
         if self.thumb_path is None:
             return False
         return self.thumb_path.exists()
 
+    @property
     def orig_exists(self) -> bool:
         if self.orig_path is None:
             return False
@@ -168,6 +171,7 @@ class Preview:
         self._widget.update(self, data)
 
     def update_progress(self, size: int, message: Soup.Message) -> None:
+        self.download_in_progress = True
         self._soup_message = message
         self._received_size += size
         if self.file_size == 0 or self._received_size == 0:
@@ -235,8 +239,13 @@ class PreviewManager:
         if urlparts.scheme in ('https', 'http'):
             if app.settings.get('preview_allow_all_images'):
                 mime_type = guess_mime_type(uri)
-                if (mime_type not in MIME_TYPES or
-                        mime_type == 'application/octet-stream'):
+                if mime_type not in MIME_TYPES:
+                    return False
+
+                if mime_type == 'application/octet-stream' and uri != oob_url:
+                    # guess_mime_type yields 'application/octet-stream' for
+                    # paths without suffix. Check oob_url to make sure we
+                    # display a preview for files sent via http upload.
                     return False
                 return True
 
@@ -291,7 +300,7 @@ class PreviewManager:
 
         preview = self._process_web_uri(uri, widget, from_us, context)
 
-        if not preview.orig_exists():
+        if not preview.orig_exists:
             if context is not None and not from_us:
                 allow_in_public = app.settings.get('preview_anonymous_muc')
                 if context == 'public' and not allow_in_public:
@@ -300,7 +309,7 @@ class PreviewManager:
 
             self.download_content(preview)
 
-        elif not preview.thumb_exists():
+        elif not preview.thumb_exists:
             load_file_async(preview.orig_path,
                             self._on_orig_load_finished,
                             preview)
@@ -343,7 +352,7 @@ class PreviewManager:
             log.error('%s: %s', preview.orig_path.name, error)
             return
 
-        preview.mime_type = guess_mime_type(preview.orig_path)
+        preview.mime_type = guess_mime_type(preview.orig_path, data)
         preview.file_size = os.path.getsize(preview.orig_path)
         if preview.is_previewable:
             if preview.create_thumbnail(data):
@@ -366,7 +375,7 @@ class PreviewManager:
             return
 
         preview.thumbnail = data
-        preview.mime_type = guess_mime_type(preview.orig_path)
+        preview.mime_type = guess_mime_type(preview.orig_path, data)
         preview.file_size = os.path.getsize(preview.orig_path)
 
         try:
@@ -385,7 +394,14 @@ class PreviewManager:
 
     def download_content(self,
                          preview: Preview,
-                         force: bool = False) -> None:
+                         force: bool = False
+                         ) -> None:
+
+        if preview.download_in_progress:
+            log.info('Download already in progress')
+            return
+
+        preview.reset_received_size()
         if preview.account is None:
             # History Window can be opened without account context
             # This means we can not apply proxy settings
@@ -461,7 +477,11 @@ class PreviewManager:
     def _on_finished(self,
                      _session: Soup.Session,
                      message: Soup.Message,
-                     preview: Preview) -> None:
+                     preview: Preview
+                     ) -> None:
+
+        preview.download_in_progress = False
+
         if message.status_code != Soup.Status.OK:
             log.warning('Download failed: %s', preview.request_uri)
             status_code = Soup.Status.get_phrase(message.status_code)
@@ -528,7 +548,10 @@ class PreviewManager:
 
         if error is not None:
             log.error('%s: %s', preview.thumb_path.name, error)
-            return
+            if not preview.thumb_exists:
+                # Generating a preview can fail if the file already exists
+                # Only abort if thumbnail has not been stored in preview
+                return
 
         log.info('Thumbnail stored: %s ', preview.thumb_path.name)
 
@@ -550,5 +573,6 @@ class PreviewManager:
         preview.update_widget(data=pixbuf)
 
     def cancel_download(self, preview: Preview) -> None:
+        preview.reset_received_size()
         session = self._get_session(preview.account)
         session.cancel_message(preview.soup_message, Soup.Status.CANCELLED)
