@@ -42,12 +42,9 @@ BoolSettings = Literal[
     'ask_online_status',
     'autoaway',
     'autoxa',
-    'change_roster_title',
     'chat_merge_consecutive_nickname',
     'check_for_update',
-    'command_system_execute',
     'confirm_close_muc',
-    'confirm_close_multiple_tabs',
     'confirm_on_window_delete',
     'confirm_paste_image',
     'dev_force_bookmark_2',
@@ -66,7 +63,6 @@ BoolSettings = Literal[
     'hide_groupchat_occupants_list',
     'ignore_incoming_attention',
     'is_window_visible',
-    'log_contact_status_changes',
     'muc_prefer_direct_msg',
     'notification_preview_message',
     'notify_on_all_muc_messages',
@@ -83,7 +79,6 @@ BoolSettings = Literal[
     'remote_control',
     'save_main_window_position',
     'send_on_ctrl_enter',
-    'shell_like_completion',
     'show_chatstate_in_banner',
     'show_help_start_chat',
     'show_in_taskbar',
@@ -122,7 +117,6 @@ IntSettings = Literal[
     'gc_sync_threshold_private_default',
     'gc_sync_threshold_public_default',
     'groupchat_roster_width',
-    'key_up_lines',
     'mainwin_height',
     'mainwin_width',
     'mainwin_x_position',
@@ -132,7 +126,6 @@ IntSettings = Literal[
     'notification_timeout',
     'preview_max_file_size',
     'preview_size',
-    'restore_timeout',
 ]
 
 StringSettings = Literal[
@@ -169,7 +162,7 @@ StringSettings = Literal[
 ]
 
 AllSettings = Literal[BoolSettings, IntSettings, StringSettings]
-AllSettingsT = Union[str, int, bool]
+AllSettingsT = Union[str, int, bool, list[str]]
 
 APP_SETTINGS = {
     'always_english_wikipedia': False,
@@ -186,15 +179,12 @@ APP_SETTINGS = {
     'autoxa': True,
     'autoxa_message': '',
     'autoxatime': 15,
-    'change_roster_title': True,
     'chat_handle_position': 350,
     'chat_merge_consecutive_nickname': True,
     'chat_timestamp_format': '%H:%M',
     'check_for_update': True,
-    'command_system_execute': False,
     'confirm_block': '',
     'confirm_close_muc': True,
-    'confirm_close_multiple_tabs': True,
     'confirm_on_window_delete': True,
     'confirm_paste_image': True,
     'dark_theme': 2,
@@ -223,14 +213,12 @@ APP_SETTINGS = {
     'hide_groupchat_banner': False,
     'hide_groupchat_occupants_list': False,
     'ignore_incoming_attention': False,
-    'key_up_lines': 25,
     'is_window_visible': True,
     'last_save_dir': '',
     'last_send_dir': '',
     'last_sounds_dir': '',
     'last_update_check': '',
     'latest_disco_addresses': '',
-    'log_contact_status_changes': False,
     'mainwin_height': 500,
     'mainwin_width': 1000,
     'mainwin_x_position': 0,
@@ -260,12 +248,10 @@ APP_SETTINGS = {
     'preview_verify_https': True,
     'print_status_in_chats': False,
     'remote_control': False,
-    'restore_timeout': -1,
     'roster_theme': 'default',
     'save_main_window_position': True,
     'search_engine': 'https://duckduckgo.com/?q=%s',
     'send_on_ctrl_enter': False,
-    'shell_like_completion': False,
     'show_chatstate_in_banner': True,
     'show_help_start_chat': True,
     'show_in_taskbar': True,
@@ -385,13 +371,13 @@ BoolGroupChatSettings = Literal[
     'notify_on_all_messages',
     'print_join_left',
     'print_status',
-    'send_chatstate',
     'send_marker',
 ]
 
 StringGroupChatSettings = Literal[
     'encryption',
     'speller_language',
+    'send_chatstate',
 ]
 
 IntGroupChatSettings = Literal[
@@ -405,6 +391,23 @@ AllGroupChatSettings = Literal[BoolGroupChatSettings,
 
 AllGroupChatSettingsT = Union[str, int, bool]
 
+
+BoolContactSettings = Literal[
+    'send_marker',
+]
+
+StringContactSettings = Literal[
+    'encryption',
+    'speller_language',
+    'send_chatstate',
+]
+
+AllContactSettings = Literal[BoolContactSettings,
+                             StringContactSettings]
+
+AllContactSettingsT = Union[str, bool]
+
+
 ACCOUNT_SETTINGS = {
     'account': {
         'account_color': 'rgb(85, 85, 85)',
@@ -499,24 +502,33 @@ StringWorkspaceSettings = Literal[
     'name',
 ]
 
-AllWorkspaceSettings = Literal[StringWorkspaceSettings, 'open_chats']
+AllWorkspaceSettings = Literal[StringWorkspaceSettings, 'chats']
 
-OpenChatSettingT = list[tuple[str, JID, str, bool]]
-AllWorkspaceSettingsT = Union[str, OpenChatSettingT]
+
+class OpenChatSettingDetails(TypedDict):
+    account: str
+    jid: JID
+    type: str
+    pinned: bool
+    position: int
+
+
+OpenChatsSettingT = list[OpenChatSettingDetails]
+AllWorkspaceSettingsT = Union[str, OpenChatsSettingT]
 
 
 class WorkspaceSettings(TypedDict):
     name: str
     color: str
     avatar_sha: str
-    open_chats: OpenChatSettingT
+    chats: OpenChatsSettingT
 
 
 WORKSPACE_SETTINGS: WorkspaceSettings = {
     'name': _('My Workspace'),
     'color': '',
     'avatar_sha': '',
-    'open_chats': [],
+    'chats': [],
 }
 
 
@@ -622,24 +634,15 @@ ADVANCED_SETTINGS = {
         'always_english_wiktionary': '',
         'ascii_formatting': _(
             'Treat * / _ pairs as possible formatting characters.'),
-        'change_roster_title': _(
-            'If enabled, Gajim will add * and [n] in contact list '
-            'window title.'),
         'chat_merge_consecutive_nickname': _(
             'Show message meta data (avatar, nickname, timestamp) only once, '
             'if there are multiple messages from the same sender within a '
             'specific timespan.'),
         'chat_timestamp_format': 'https://docs.python.org/3/library/time.html#time.strftime',  # noqa: E501
-        'command_system_execute': _(
-            'If enabled, Gajim will execute commands '
-            '(/show, /sh, /execute, /exec).'),
         'confirm_block': _(
             'Show a confirmation dialog to block a contact? Empty string '
             'means never show the dialog.'),
         'confirm_close_muc': _('Ask before closing a group chat tab/window.'),
-        'confirm_close_multiple_tabs': _(
-            'Ask before closing tabbed chat window if there are chats that '
-            'can lose data (chat, private chat).'),
         'confirm_on_window_delete': _(
             'Ask before quitting when Gajim’s window is closed'),
         'confirm_paste_image': _('Ask before pasting an image.'),
@@ -669,15 +672,9 @@ ADVANCED_SETTINGS = {
         'groupchat_roster_width': _('Width of group chat roster in pixel'),
         'hide_chat_banner': _('Hides the banner in a 1:1 chat window.'),
         'hide_groupchat_banner': _('Hides the banner in a group chat window.'),
-        'hide_groupchat_occupants_list': _(
-            'Hides the group chat participants list in a group chat window.'),
         'ignore_incoming_attention': _(
             'If enabled, Gajim will ignore incoming attention '
             'requests ("wizz").'),
-        'key_up_lines': _(
-            'How many lines to store for Ctrl+KeyUP '
-            '(previously sent messages).'),
-        'log_contact_status_changes': '',
         'muc_highlight_words': _(
             'A list of words (semicolon separated) that will be '
             'highlighted in group chats.'),
@@ -691,18 +688,12 @@ ADVANCED_SETTINGS = {
         'notify_on_all_muc_messages': '',
         'plugins_repository_enabled': _(
             'If enabled, Gajim offers to download plugins hosted on gajim.org'),
-        'restore_timeout': _(
-            'How far back in time (minutes) chat history is restored. -1 '
-            'means no limit.'),
         'save_main_window_position': _(
             'If enabled, Gajim will save the main window position when hiding '
             'it, and restore it when showing the window again.'),
         'search_engine': '',
         'send_on_ctrl_enter': _(
             'Send message on Ctrl+Enter and make a new line with Enter.'),
-        'shell_like_completion': _(
-            'If enabled, completion in group chats will be like a shell '
-            'auto-completion.'),
         'stun_server': _('STUN server to use when using Jingle'),
         'trayicon_notification_on_events': _(
             'Notify of events in the notification area.'),
