@@ -13,12 +13,19 @@
 # along with Gajim. If not, see <http://www.gnu.org/licenses/>.
 
 from typing import Any
+from typing import Callable
+from typing import Literal
+from typing import NamedTuple
+from typing import Optional
 from typing import Dict
 from typing import List
 from typing import Union
+from typing import overload
+from typing import cast
 
 import sys
 import json
+import uuid
 import logging
 import sqlite3
 import inspect
@@ -28,27 +35,54 @@ from collections import namedtuple
 from collections import defaultdict
 
 from gi.repository import GLib
+from nbxmpp.protocol import JID
 
 from gajim import IS_PORTABLE
 from gajim.common import app
 from gajim.common import configpaths
 from gajim.common import optparser
-from gajim.common.helpers import get_muc_context
+from gajim.common.storage.base import Encoder
+from gajim.common.storage.base import json_decoder
 from gajim.common.setting_values import APP_SETTINGS
+from gajim.common.setting_values import AllContactSettings
+from gajim.common.setting_values import AllContactSettingsT
+from gajim.common.setting_values import BoolContactSettings
+from gajim.common.setting_values import StringContactSettings
+from gajim.common.setting_values import AllGroupChatSettings
+from gajim.common.setting_values import AllGroupChatSettingsT
+from gajim.common.setting_values import BoolGroupChatSettings
+from gajim.common.setting_values import IntGroupChatSettings
+from gajim.common.setting_values import StringGroupChatSettings
+from gajim.common.setting_values import AllWorkspaceSettings
+from gajim.common.setting_values import AllWorkspaceSettingsT
+from gajim.common.setting_values import OpenChatsSettingT
+from gajim.common.setting_values import StringWorkspaceSettings
 from gajim.common.setting_values import ACCOUNT_SETTINGS
 from gajim.common.setting_values import PROXY_SETTINGS
 from gajim.common.setting_values import PROXY_EXAMPLES
+from gajim.common.setting_values import INITAL_WORKSPACE
 from gajim.common.setting_values import PLUGIN_SETTINGS
+from gajim.common.setting_values import WORKSPACE_SETTINGS
 from gajim.common.setting_values import DEFAULT_SOUNDEVENT_SETTINGS
 from gajim.common.setting_values import STATUS_PRESET_SETTINGS
 from gajim.common.setting_values import STATUS_PRESET_EXAMPLES
 from gajim.common.setting_values import HAS_APP_DEFAULT
 from gajim.common.setting_values import HAS_ACCOUNT_DEFAULT
+from gajim.common.setting_values import AllSettingsT
+from gajim.common.setting_values import IntSettings
+from gajim.common.setting_values import IntAccountSettings
+from gajim.common.setting_values import StringSettings
+from gajim.common.setting_values import StringAccountSettings
+from gajim.common.setting_values import BoolSettings
+from gajim.common.setting_values import BoolAccountSettings
 
 SETTING_TYPE = Union[bool, int, str, object]
 
+
 log = logging.getLogger('gajim.c.settings')
 
+CURRENT_USER_VERSION = 2
+
 CREATE_SQL = '''
     CREATE TABLE settings (
             name TEXT UNIQUE,
@@ -65,33 +99,50 @@ CREATE_SQL = '''
     INSERT INTO settings(name, settings) VALUES ('status_presets', '%s');
     INSERT INTO settings(name, settings) VALUES ('proxies', '%s');
     INSERT INTO settings(name, settings) VALUES ('plugins', '{}');
+    INSERT INTO settings(name, settings) VALUES ('workspaces', '%s');
 
-    PRAGMA user_version=0;
+    PRAGMA user_version=%s;
     ''' % (json.dumps(STATUS_PRESET_EXAMPLES),
-           json.dumps(PROXY_EXAMPLES))
+           json.dumps(PROXY_EXAMPLES),
+           json.dumps(INITAL_WORKSPACE),
+           CURRENT_USER_VERSION)
+
+
+_SignalCallable = Callable[[Any, str, Optional[str], Optional[JID]], Any]
+_CallbackDict = dict[tuple[str, Optional[str], Optional[JID]],
+                     list[weakref.WeakMethod[_SignalCallable]]]
+
+if app.is_flatpak():
+    OVERRIDES_PATH = Path('/app/app-overrides.json')
+else:
+    OVERRIDES_PATH = Path('/etc/gajim/app-overrides.json')
 
 
 class Settings:
     def __init__(self):
-        self._con = None
+        self._con = cast(sqlite3.Connection, None)
         self._commit_scheduled = None
 
         self._settings = {}
+        self._app_overrides: dict[str, AllSettingsT] = {}
         self._account_settings = {}
 
-        self._callbacks = defaultdict(list)
+        self._callbacks: _CallbackDict = defaultdict(list)
 
-    def connect_signal(self, setting, func, account=None, jid=None):
+    def connect_signal(self,
+                       setting: str,
+                       func: _SignalCallable,
+                       account: Optional[str] = None,
+                       jid: Optional[JID] = None) -> None:
         if not inspect.ismethod(func):
             # static methods are not bound to an object so we can’t easily
             # remove the func once it should not be called anymore
             raise ValueError('Only bound methods can be connected')
 
+        weak_func = weakref.WeakMethod(func)
+        self._callbacks[(setting, account, jid)].append(weak_func)
 
-        func = weakref.WeakMethod(func)
-        self._callbacks[(setting, account, jid)].append(func)
-
-    def disconnect_signals(self, object_):
+    def disconnect_signals(self, object_: object) -> Any:
         for _, handlers in self._callbacks.items():
             for handler in list(handlers):
                 if isinstance(handler, tuple):
@@ -101,24 +152,30 @@ class Settings:
                     handlers.remove(handler)
 
     def bind_signal(self,
-                    setting,
-                    widget,
-                    func_name,
-                    account=None,
-                    jid=None,
-                    inverted=False,
-                    default_text=None):
+                    setting: str,
+                    widget: Any,
+                    func_name: str,
+                    account: Optional[str] = None,
+                    jid: Optional[JID] = None,
+                    inverted: bool = False,
+                    default_text: Optional[str] = None
+                    ) -> None:
 
         callbacks = self._callbacks[(setting, account, jid)]
         func = getattr(widget, func_name)
         callbacks.append((func, inverted, default_text))
 
-        def _on_destroy(*args):
+        def _on_destroy(*args: Any) -> None:
             callbacks.remove((func, inverted, default_text))
 
         widget.connect('destroy', _on_destroy)
 
-    def _notify(self, value, setting, account=None, jid=None):
+    def _notify(self,
+                value: Any,
+                setting: str,
+                account: Optional[str] = None,
+                jid: Optional[JID] = None) -> None:
+
         log.info('Signal: %s changed', setting)
 
         callbacks = self._callbacks[(setting, account, jid)]
@@ -156,19 +213,42 @@ class Settings:
         self._load_settings()
         self._load_account_settings()
         if not self._settings['app']:
+            # ['app'] is empty in a newly created database.
+            # If there is an old config, it gets migrated at this point.
             self._migrate_old_config()
             self._commit()
         self._migrate_database()
+        self._load_app_overrides()
+        self._commit()
 
     @staticmethod
     def _setup_installation_defaults() -> None:
         if IS_PORTABLE:
             APP_SETTINGS['use_keyring'] = False
 
+        if sys.platform in ('win32', 'darwin'):
+            # Cairo does not support color emojis fonts on
+            # Windows and MacOS
+            APP_SETTINGS['dev_use_message_label'] = False
+
+    def _load_app_overrides(self) -> None:
+        if not OVERRIDES_PATH.exists():
+            return
+
+        with OVERRIDES_PATH.open(encoding='utf8') as f:
+            try:
+                self._app_overrides = json.load(f)
+            except Exception:
+                log.exception('Failed to load overrides')
+                return
+
+        self._settings['app'].update(self._app_overrides)
+
     @staticmethod
-    def _namedtuple_factory(cursor: Any, row: Any) -> Any:
+    def _namedtuple_factory(cursor: sqlite3.Cursor,
+                            row: tuple[Any, ...]) -> NamedTuple:
         fields = [col[0] for col in cursor.description]
-        return namedtuple("Row", fields)(*row)
+        return namedtuple('Row', fields)(*row)  # type: ignore
 
     def _connect_database(self) -> None:
         path = configpaths.get('SETTINGS')
@@ -235,7 +315,30 @@ class Settings:
             sys.exit()
 
     def _migrate(self) -> None:
-        pass
+        version = self._get_user_version()
+        if version < 1:
+            sql = '''INSERT INTO settings(name, settings)
+                     VALUES ('workspaces', ?)'''
+            self._con.execute(sql, (json.dumps(INITAL_WORKSPACE),))
+            self._settings['workspaces'] = INITAL_WORKSPACE
+            self._set_user_version(1)
+
+        if version < 2:
+            # Migrate open chats to new key and format
+            for workspace in self._settings['workspaces'].values():
+                open_chats = []
+                for open_chat in workspace.get('open_chats', []):
+                    account, jid, type_, pinned = open_chat
+                    open_chats.append({'account': account,
+                                       'jid': jid,
+                                       'type': type_,
+                                       'pinned': pinned,
+                                       'position': -1})
+
+                workspace['chats'] = open_chats
+                workspace.pop('open_chats', None)
+
+            self._set_user_version(2)
 
     def _migrate_old_config(self) -> None:
         config_file = configpaths.get('CONFIG_FILE')
@@ -256,6 +359,9 @@ class Settings:
                                              group_chat_settings)
 
         for account, settings in account_settings.items():
+            if account == 'Local':
+                # Zeroconf support was dropped so don’t migrate the account
+                continue
             self.add_account(account)
             self._account_settings[account]['account'] = settings
             self._account_settings[account]['contact'] = contact_settings
@@ -283,13 +389,13 @@ class Settings:
         # Migrate deprecated settings
         value = app_settings.pop('send_chatstate_muc_default', None)
         if value is not None:
-            for account in self._account_settings:
-                self._account_settings[account]['account']['gc_send_chatstate_default'] = value
+            for account, settings in self._account_settings.items():
+                settings['account']['gc_send_chatstate_default'] = value
 
         value = app_settings.pop('send_chatstate_default', None)
         if value is not None:
-            for account in self._account_settings:
-                self._account_settings[account]['account']['send_chatstate_default'] = value
+            for account, settings in self._account_settings.items():
+                settings['account']['send_chatstate_default'] = value
 
         value = app_settings.pop('print_join_left_default', None)
         if value is not None:
@@ -336,7 +442,7 @@ class Settings:
             else:
                 category = 'contact'
 
-            if not jid in self._account_settings[account][category]:
+            if jid not in self._account_settings[account][category]:
                 self._account_settings[account][category][jid] = {
                     'encryption': encryption}
             else:
@@ -344,7 +450,8 @@ class Settings:
                     jid]['encryption'] = encryption
             self._commit_account_settings(account)
 
-    def _split_encryption_config_key(self, key: str) -> Any:
+    def _split_encryption_config_key(self, key: str) -> tuple[Optional[str],
+                                                              Optional[str]]:
         for account in self._account_settings:
             if not key.startswith(account):
                 continue
@@ -418,20 +525,23 @@ class Settings:
         log.info('Close settings')
         self._con.commit()
         self._con.close()
-        self._con = None
+        self._con = cast(sqlite3.Connection, None)
 
     def _load_settings(self) -> None:
         settings = self._con.execute('SELECT * FROM settings').fetchall()
         for row in settings:
             log.info('Load %s settings', row.name)
-            self._settings[row.name] = json.loads(row.settings)
+            self._settings[row.name] = json.loads(row.settings,
+                                                  object_hook=json_decoder)
 
     def _load_account_settings(self) -> None:
         account_settings = self._con.execute(
             'SELECT * FROM account_settings').fetchall()
         for row in account_settings:
             log.info('Load account settings: %s', row.account)
-            self._account_settings[row.account] = json.loads(row.settings)
+            self._account_settings[row.account] = json.loads(
+                row.settings,
+                object_hook=json_decoder)
 
     def _commit_account_settings(self,
                                  account: str,
@@ -439,7 +549,7 @@ class Settings:
         log.info('Set account settings: %s', account)
         self._con.execute(
             'UPDATE account_settings SET settings = ? WHERE account = ?',
-            (json.dumps(self._account_settings[account]), account))
+            (json.dumps(self._account_settings[account], cls=Encoder), account))
 
         self._commit(schedule=schedule)
 
@@ -447,11 +557,26 @@ class Settings:
         log.info('Set settings: %s', name)
         self._con.execute(
             'UPDATE settings SET settings = ? WHERE name = ?',
-            (json.dumps(self._settings[name]), name))
+            (json.dumps(self._settings[name], cls=Encoder), name))
 
         self._commit(schedule=schedule)
 
-    def get_app_setting(self, setting: str) -> SETTING_TYPE:
+    def has_app_override(self, setting: str) -> bool:
+        return setting in self._app_overrides
+
+    @overload
+    def get_app_setting(self, setting: BoolSettings) -> bool:
+        ...
+
+    @overload
+    def get_app_setting(self, setting: StringSettings) -> str:
+        ...
+
+    @overload
+    def get_app_setting(self, setting: IntSettings) -> int:
+        ...
+
+    def get_app_setting(self, setting: str) -> AllSettingsT:
         if setting not in APP_SETTINGS:
             raise ValueError(f'Invalid app setting: {setting}')
 
@@ -462,10 +587,42 @@ class Settings:
 
     get = get_app_setting
 
-    def set_app_setting(self, setting: str, value: SETTING_TYPE) -> None:
+    @overload
+    def set_app_setting(self,
+                        setting: BoolSettings,
+                        value: Optional[bool]) -> None:
+        ...
+
+    @overload
+    def set_app_setting(self,
+                        setting: StringSettings,
+                        value: Optional[str]) -> None:
+        ...
+
+    @overload
+    def set_app_setting(self,
+                        setting: IntSettings,
+                        value: Optional[int]) -> None:
+        ...
+
+    @overload
+    def set_app_setting(self,
+                        setting: Literal['workspace_order'],
+                        value: list[str]) -> None:
+        ...
+
+    def set_app_setting(self,
+                        setting: str,
+                        value: Optional[AllSettingsT]) -> None:
+
         if setting not in APP_SETTINGS:
             raise ValueError(f'Invalid app setting: {setting}')
 
+        if setting in self._app_overrides:
+            log.warning('Changing %s is not allowed because there exists an '
+                        'override', setting)
+            return
+
         default = APP_SETTINGS[setting]
         if not isinstance(value, type(default)) and value is not None:
             raise TypeError(f'Invalid type for {setting}: '
@@ -488,7 +645,7 @@ class Settings:
 
     set = set_app_setting
 
-    def get_plugin_setting(self, plugin: str, setting: str) ->  SETTING_TYPE:
+    def get_plugin_setting(self, plugin: str, setting: str) -> SETTING_TYPE:
         if setting not in PLUGIN_SETTINGS:
             raise ValueError(f'Invalid plugin setting: {setting}')
 
@@ -552,9 +709,34 @@ class Settings:
     def get_accounts(self) -> List[str]:
         return list(self._account_settings.keys())
 
+    def get_active_accounts(self) -> List[str]:
+        active = []
+        for account in self._account_settings:
+            if self.get_account_setting(account, 'active'):
+                active.append(account)
+        return active
+
+    @overload
     def get_account_setting(self,
                             account: str,
-                            setting: str) -> SETTING_TYPE:
+                            setting: StringAccountSettings) -> str:
+        ...
+
+    @overload
+    def get_account_setting(self,
+                            account: str,
+                            setting: IntAccountSettings) -> int:
+        ...
+
+    @overload
+    def get_account_setting(self,
+                            account: str,
+                            setting: BoolAccountSettings) -> bool:
+        ...
+
+    def get_account_setting(self,
+                            account: str,
+                            setting: str) -> AllSettingsT:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -567,10 +749,31 @@ class Settings:
         except KeyError:
             return ACCOUNT_SETTINGS['account'][setting]
 
+    @overload
+    def set_account_setting(self,
+                            account: str,
+                            setting: StringAccountSettings,
+                            value: Optional[str]) -> None:
+        ...
+
+    @overload
+    def set_account_setting(self,
+                            account: str,
+                            setting: IntAccountSettings,
+                            value: Optional[int]) -> None:
+        ...
+
+    @overload
+    def set_account_setting(self,
+                            account: str,
+                            setting: BoolAccountSettings,
+                            value: Optional[bool]) -> None:
+        ...
+
     def set_account_setting(self,
                             account: str,
                             setting: str,
-                            value: SETTING_TYPE) -> None:
+                            value: Optional[AllSettingsT]) -> None:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -598,10 +801,35 @@ class Settings:
         self._commit_account_settings(account)
         self._notify(value, setting, account)
 
+    @overload
     def get_group_chat_setting(self,
                                account: str,
-                               jid: str,
-                               setting: str) -> SETTING_TYPE:
+                               jid: JID,
+                               setting: IntGroupChatSettings
+                               ) -> int:
+        ...
+
+    @overload
+    def get_group_chat_setting(self,
+                               account: str,
+                               jid: JID,
+                               setting: BoolGroupChatSettings
+                               ) -> bool:
+        ...
+
+    @overload
+    def get_group_chat_setting(self,
+                               account: str,
+                               jid: JID,
+                               setting: StringGroupChatSettings
+                               ) -> str:
+        ...
+
+    def get_group_chat_setting(self,
+                               account: str,
+                               jid: JID,
+                               setting: AllGroupChatSettings
+                               ) -> AllGroupChatSettingsT:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -613,7 +841,9 @@ class Settings:
             return self._account_settings[account]['group_chat'][jid][setting]
         except KeyError:
 
-            context = get_muc_context(jid)
+            client = app.get_client(account)
+            contact = client.get_module('Contacts').get_contact(jid)
+            context = contact.muc_context
             if context is None:
                 # If there is no disco info available
                 # to determine the context assume public
@@ -637,11 +867,35 @@ class Settings:
 
             return default
 
+    @overload
     def set_group_chat_setting(self,
                                account: str,
-                               jid: str,
-                               setting: str,
-                               value: SETTING_TYPE) -> None:
+                               jid: JID,
+                               setting: StringGroupChatSettings,
+                               value: str) -> None:
+        ...
+
+    @overload
+    def set_group_chat_setting(self,
+                               account: str,
+                               jid: JID,
+                               setting: IntGroupChatSettings,
+                               value: int) -> None:
+        ...
+
+    @overload
+    def set_group_chat_setting(self,
+                               account: str,
+                               jid: JID,
+                               setting: BoolGroupChatSettings,
+                               value: bool) -> None:
+        ...
+
+    def set_group_chat_setting(self,
+                               account: str,
+                               jid: JID,
+                               setting: AllGroupChatSettings,
+                               value: AllGroupChatSettingsT) -> None:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -652,7 +906,9 @@ class Settings:
         default = ACCOUNT_SETTINGS['group_chat'][setting]
         if default in (HAS_APP_DEFAULT, HAS_ACCOUNT_DEFAULT):
 
-            context = get_muc_context(jid)
+            client = app.get_client(account)
+            contact = client.get_module('Contacts').get_contact(jid)
+            context = contact.muc_context
             if context is None:
                 # If there is no disco info available
                 # to determine the context assume public
@@ -663,11 +919,9 @@ class Settings:
             if default is HAS_ACCOUNT_DEFAULT:
                 default_store = ACCOUNT_SETTINGS['account']
 
-            context_default_setting = f'gc_{setting}_{context}_default'
-            if context_default_setting in default_store:
-                default = default_store[context_default_setting]
-            else:
-                default = default_store[f'gc_{setting}_default']
+            default = default_store.get(f'gc_{setting}_{context}_default')
+            if default is None:
+                default = default_store.get(f'gc_{setting}_default')
 
         if not isinstance(value, type(default)) and value is not None:
             raise TypeError(f'Invalid type for {setting}: '
@@ -685,7 +939,7 @@ class Settings:
 
         group_chat_settings = self._account_settings[account]['group_chat']
         if jid not in group_chat_settings:
-            group_chat_settings[jid] = {setting: value}
+            group_chat_settings[str(jid)] = {setting: value}
         else:
             group_chat_settings[jid][setting] = value
 
@@ -697,17 +951,36 @@ class Settings:
                                 value: SETTING_TYPE,
                                 context: str = None) -> None:
 
-        for account in self._account_settings:
-            for jid in self._account_settings[account]['group_chat']:
+        for account, acc_settings in self._account_settings.items():
+            for jid in acc_settings['group_chat']:
                 if context is not None:
-                    if get_muc_context(jid) != context:
+                    client = app.get_client(account)
+                    contact = client.get_module('Contacts').get_contact(jid)
+                    if contact.muc_context != context:
                         continue
                 self.set_group_chat_setting(account, jid, setting, value)
 
+    @overload
     def get_contact_setting(self,
                             account: str,
-                            jid: str,
-                            setting: str) -> SETTING_TYPE:
+                            jid: JID,
+                            setting: BoolContactSettings
+                            ) -> bool:
+        ...
+
+    @overload
+    def get_contact_setting(self,
+                            account: str,
+                            jid: JID,
+                            setting: StringContactSettings
+                            ) -> str:
+        ...
+
+    def get_contact_setting(self,
+                            account: str,
+                            jid: JID,
+                            setting: AllContactSettings
+                            ) -> AllContactSettingsT:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -727,11 +1000,27 @@ class Settings:
 
             return default
 
+    @overload
     def set_contact_setting(self,
                             account: str,
-                            jid: str,
-                            setting: str,
-                            value: SETTING_TYPE) -> None:
+                            jid: JID,
+                            setting: StringContactSettings,
+                            value: str) -> None:
+        ...
+
+    @overload
+    def set_contact_setting(self,
+                            account: str,
+                            jid: JID,
+                            setting: BoolContactSettings,
+                            value: bool) -> None:
+        ...
+
+    def set_contact_setting(self,
+                            account: str,
+                            jid: JID,
+                            setting: AllContactSettings,
+                            value: AllContactSettingsT) -> None:
 
         if account not in self._account_settings:
             raise ValueError(f'Account missing: {account}')
@@ -764,7 +1053,7 @@ class Settings:
 
         contact_settings = self._account_settings[account]['contact']
         if jid not in contact_settings:
-            contact_settings[jid] = {setting: value}
+            contact_settings[str(jid)] = {setting: value}
         else:
             contact_settings[jid][setting] = value
 
@@ -775,8 +1064,8 @@ class Settings:
                              setting: str,
                              value: SETTING_TYPE) -> None:
 
-        for account in self._account_settings:
-            for jid in self._account_settings[account]['contact']:
+        for account, acc_settings in self._account_settings.items():
+            for jid in acc_settings['contact']:
                 self.set_contact_setting(account, jid, setting, value)
 
     def set_soundevent_setting(self,
@@ -907,6 +1196,96 @@ class Settings:
             if self.get_account_setting(account, 'proxy') == proxy_name:
                 self.set_account_setting(account, 'proxy', None)
 
+    @overload
+    def set_workspace_setting(self,
+                              workspace_id: str,
+                              setting: StringWorkspaceSettings,
+                              value: str) -> None:
+        ...
+
+    @overload
+    def set_workspace_setting(self,
+                              workspace_id: str,
+                              setting: Literal['chats'],
+                              value: OpenChatsSettingT
+                              ) -> None:
+        ...
+
+    def set_workspace_setting(self,
+                              workspace_id: str,
+                              setting: AllWorkspaceSettings,
+                              value: AllWorkspaceSettingsT) -> None:
+
+        if setting not in WORKSPACE_SETTINGS:
+            raise ValueError(f'Invalid workspace setting: {setting}')
+
+        if workspace_id not in self._settings['workspaces']:
+            raise ValueError(f'Workspace does not exists: {workspace_id}')
+
+        default = WORKSPACE_SETTINGS[setting]
+        if not isinstance(value, type(default)):
+            raise TypeError(f'Invalid type for {setting}: '
+                            f'{value} {type(value)}')
+
+        self._settings['workspaces'][workspace_id][setting] = value
+        self._commit_settings('workspaces')
+
+    @overload
+    def get_workspace_setting(self,
+                              workspace_id: str,
+                              setting: Literal['chats']
+                              ) -> OpenChatsSettingT:
+        ...
+
+    @overload
+    def get_workspace_setting(self,
+                              workspace_id: str,
+                              setting: StringWorkspaceSettings) -> str:
+        ...
+
+    def get_workspace_setting(self,
+                              workspace_id: str,
+                              setting: AllWorkspaceSettings
+                              ) -> AllWorkspaceSettingsT:
+
+        if setting not in WORKSPACE_SETTINGS:
+            raise ValueError(f'Invalid workspace setting: {setting}')
+
+        if workspace_id not in self._settings['workspaces']:
+            raise ValueError(f'Workspace does not exists: {workspace_id}')
+
+        try:
+            return self._settings['workspaces'][workspace_id][setting]
+        except KeyError:
+            return WORKSPACE_SETTINGS[setting]
+
+    def get_workspaces(self) -> List[str]:
+        workspace_order = app.settings.get_app_setting('workspace_order')
+
+        def sort_workspaces(workspace_id):
+            try:
+                return workspace_order.index(workspace_id)
+            except ValueError:
+                # Handle the case that a workspace id is for some reason not
+                # in the workspace order list
+                return 10000
+
+        workspaces = list(self._settings['workspaces'].keys())
+        workspaces.sort(key=sort_workspaces)
+        return workspaces
+
+    def add_workspace(self, name: str) -> str:
+        id_ = str(uuid.uuid4())
+        self._settings['workspaces'][id_] = {
+            'name': name,
+        }
+        self._commit_settings('workspaces')
+        return id_
+
+    def remove_workspace(self, id_: str) -> None:
+        del self._settings['workspaces'][id_]
+        self._commit_settings('workspaces')
+
 
 class LegacyConfig:
 
