@@ -85,7 +85,7 @@ class SocksQueue:
         log.debug('Start listening for socks5 connection')
         sid = file_props.sid
         self.sha_handlers[sha_str] = (sha_handler, sid)
-        if self.listener is None or self.listener.connections == []:
+        if self.listener is None or not self.listener.connections:
             self.listener = Socks5Listener(self.idlequeue, port, file_props,
                 fingerprint=fingerprint)
             self.listener.queue = self
@@ -333,7 +333,7 @@ class SocksQueue:
 
     def isHashInSockObjs(self, sockobjs, hash_):
         '''
-        It tells wether there is a particular hash in sockobjs or not
+        It tells whether there is a particular hash in sockobjs or not
         '''
         for key in sockobjs:
             if hash_ in key:
@@ -341,7 +341,7 @@ class SocksQueue:
         return False
 
     def on_connection_accepted(self, sock, listener):
-        sock_hash = sock.__hash__()
+        sock_hash = hash(sock)
         if listener.file_props.type_ == 's' and \
         not self.isHashInSockObjs(self.senders, sock_hash):
             sockobj = Socks5SenderServer(self.idlequeue, sock_hash, self,
@@ -683,15 +683,20 @@ class Socks5:
             lenn = 0
             try:
                 lenn = self._send(buff)
-            except (OpenSSL.SSL.WantReadError, OpenSSL.SSL.WantWriteError,
-            OpenSSL.SSL.WantX509LookupError) as e:
-                log.info('SSL rehandshake request: %s', repr(e))
-                raise e
+            except (OpenSSL.SSL.WantReadError,
+                    OpenSSL.SSL.WantWriteError,
+                    OpenSSL.SSL.WantX509LookupError) as err:
+                log.info('SSL rehandshake request: %s', repr(err))
+                raise err
             except OpenSSL.SSL.SysCallError:
                 return self._on_send_exception()
-            except Exception as e:
-                if e.errno not in (EINTR, ENOBUFS, EWOULDBLOCK):
+            except socket.error as err:
+                if err.errno not in (EINTR, ENOBUFS, EWOULDBLOCK):
                     return self._on_send_exception()
+            except Exception as err:
+                log.error(err)
+                return self._on_send_exception()
+
             self.size += lenn
             current_time = time.time()
             self.file_props.elapsed_time += current_time - \
@@ -712,10 +717,10 @@ class Socks5:
                 return None
             self.file_props.stalled = False
             return lenn
-        else:
-            self.state = 8 # end connection
-            self.disconnect()
-            return -1
+
+        self.state = 8 # end connection
+        self.disconnect()
+        return -1
 
     def _on_send_exception(self):
         # peer stopped reading
@@ -939,6 +944,7 @@ class Socks5Sender(IdleObject):
     """
     def __init__(self, idlequeue, sock_hash, parent, _sock, host=None,
                  port=None, fingerprint=None, connected=True, file_props=None):
+        IdleObject.__init__(self)
         self.fingerprint = fingerprint
         self.queue_idx = sock_hash
         self.queue = parent
@@ -1023,6 +1029,7 @@ class Socks5Receiver(IdleObject):
         fingerprint: fingerprint of certificates we shall use, set to None if
         TLS connection not desired
         """
+        IdleObject.__init__(self)
         self.queue_idx = -1
         self.streamhost = streamhost
         self.queue = None
@@ -1297,8 +1304,7 @@ class Socks5Client(Socks5):
                     return
                 if self.state < 5:
                     self.idlequeue.set_read_timeout(self.fd, CONNECT_TIMEOUT)
-                    result = self.main(0)
-                    self.queue.process_result(result, self)
+                    self.main(0)
                 elif self.state == 5: # wait for proxy reply
                     pass
                 elif self.file_props.type_ == 'r':
@@ -1395,7 +1401,7 @@ class Socks5ReceiverServer(Socks5Server, Socks5Receiver):
 class Socks5Listener(IdleObject):
     def __init__(self, idlequeue, port, fp, fingerprint=None):
         """
-        Handle all incomming connections on (0.0.0.0, port)
+        Handle all incoming connections on (0.0.0.0, port)
 
         This class implements IdleObject, but we will expect
         only pollin events though
@@ -1403,6 +1409,7 @@ class Socks5Listener(IdleObject):
         fingerprint: fingerprint of certificates we shall use, set to None if
         TLS connection not desired
         """
+        IdleObject.__init__(self)
         self.port = port
         self.ais = socket.getaddrinfo(None, port, socket.AF_UNSPEC,
             socket.SOCK_STREAM, socket.SOL_TCP, socket.AI_PASSIVE)
@@ -1465,7 +1472,7 @@ class Socks5Listener(IdleObject):
 
     def pollin(self):
         """
-        Accept a new incomming connection and notify queue
+        Accept a new incoming connection and notify queue
         """
         sock = self.accept_conn()
         self.queue.on_connection_accepted(sock, self)
@@ -1486,7 +1493,7 @@ class Socks5Listener(IdleObject):
 
     def accept_conn(self):
         """
-        Accept a new incomming connection
+        Accept a new incoming connection
         """
         _sock = self._serv.accept()
         _sock[0].setblocking(False)
