@@ -42,9 +42,10 @@ from nbxmpp.structs import MessageProperties
 from gajim.common import app
 from gajim.common import configpaths
 from gajim.common.helpers import AdditionalDataDict
-from gajim.common.const import ShowConstant
+from gajim.common.const import MAX_MESSAGE_CORRECTION_DELAY
 from gajim.common.const import KindConstant
 from gajim.common.const import JIDConstant
+from gajim.common.modules.contacts import GroupchatContact
 
 from gajim.common.storage.base import SqliteStorage
 from gajim.common.storage.base import timeit
@@ -178,7 +179,6 @@ class MessageArchiveStorage(SqliteStorage):
         self._con.row_factory = self._namedtuple_factory
 
         self._con.create_function('like', 1, self._like)
-        self._con.create_function('get_timeout', 0, self._get_timeout)
 
         self._get_jid_ids_from_db()
 
@@ -222,7 +222,7 @@ class MessageArchiveStorage(SqliteStorage):
                     )''',
 
                 '''CREATE INDEX IF NOT EXISTS idx_logs_stanza_id
-                    ON logs(stanza_id)''',
+                   ON logs(stanza_id)''',
                 'PRAGMA user_version=1'
             ]
 
@@ -252,23 +252,12 @@ class MessageArchiveStorage(SqliteStorage):
 
         if user_version < 5:
             statements = [
-                'CREATE INDEX idx_logs_message_id ON logs (message_id)',
+                '''CREATE INDEX IF NOT EXISTS idx_logs_message_id
+                   ON logs (message_id)''',
                 'PRAGMA user_version=5'
             ]
             self._execute_multiple(statements)
 
-    @staticmethod
-    def _get_timeout() -> int:
-        '''
-        returns the timeout in epoch
-        '''
-        timeout = app.settings.get('restore_timeout')
-
-        now = int(time.time())
-        if timeout > 0:
-            timeout = now - (timeout * 60)
-        return timeout
-
     @staticmethod
     def _like(search_str: str) -> str:
         return f'%{search_str}%'
@@ -351,31 +340,6 @@ class MessageArchiveStorage(SqliteStorage):
         self._delayed_commit()
         return lastrowid
 
-    @staticmethod
-    def convert_show_values_to_db_api_values(show: Optional[str]
-                                             ) -> Optional[ShowConstant]:
-        '''
-        Convert from string style to constant ints for db
-        '''
-
-        if show == 'online':
-            return ShowConstant.ONLINE
-        if show == 'chat':
-            return ShowConstant.CHAT
-        if show == 'away':
-            return ShowConstant.AWAY
-        if show == 'xa':
-            return ShowConstant.XA
-        if show == 'dnd':
-            return ShowConstant.DND
-        if show == 'offline':
-            return ShowConstant.OFFLINE
-        if show is None:
-            return ShowConstant.ONLINE
-        # invisible in GC when someone goes invisible
-        # it's a RFC violation .... but we should not crash
-        return None
-
     @timeit
     def get_conversation_before_after(self,
                                       account: str,
@@ -868,6 +832,38 @@ class MessageArchiveStorage(SqliteStorage):
             sql,
             tuple(jids) + (date_ts, delta_ts)).fetchone()
 
+    @timeit
+    def get_recent_muc_nicks(self, contact: GroupchatContact) -> list[str]:
+        '''
+        Queries the last 50 message rows and gathers nicknames in a list
+        '''
+        jids = [contact.jid]
+        account_id = self.get_account_id(contact.account)
+        kinds = map(str, [KindConstant.STATUS,
+                          KindConstant.GCSTATUS,
+                          KindConstant.ERROR])
+
+        sql = '''
+            SELECT contact_name
+            FROM logs NATURAL JOIN jids WHERE jid IN ({jids})
+            AND account_id = {account_id}
+            AND kind NOT IN ({kinds})
+            ORDER BY time DESC
+            '''.format(jids=', '.join('?' * len(jids)),
+                       account_id=account_id,
+                       kinds=', '.join(kinds))
+
+        result = self._con.execute(
+            sql,
+            tuple(jids)).fetchmany(50)
+
+        nicknames: list[str] = []
+        for row in result:
+            if row.contact_name not in nicknames:
+                nicknames.append(row.contact_name)
+
+        return nicknames
+
     @timeit
     def deduplicate_muc_message(self,
                                 account: str,
@@ -993,29 +989,86 @@ class MessageArchiveStorage(SqliteStorage):
         return False
 
     @timeit
-    def store_message_correction(self,
-                                 account: str,
-                                 jid: JID,
-                                 correct_id: str,
-                                 corrected_text: str,
-                                 is_groupchat: bool) -> None:
-        type_ = JIDConstant.NORMAL_TYPE
-        if is_groupchat:
-            type_ = JIDConstant.ROOM_TYPE
-
-        jid_id = self.get_jid_id(str(jid), type_=type_)
+    def get_last_correctable_message(self,
+                                     account: str,
+                                     jid: JID,
+                                     message_id: str
+                                     ) -> Optional[LastConversationRow]:
+        '''
+        Load the last correctable message of a conversation by message_id.
+        Conditions: max 5 min old
+        '''
+        jids = [jid]
         account_id = self.get_account_id(account)
+        min_time = time.time() - MAX_MESSAGE_CORRECTION_DELAY
+
         sql = '''
-            SELECT log_line_id, message, additional_data
-            FROM logs
-            WHERE +jid_id = ?
-            AND account_id = ?
+            SELECT contact_name, time, kind, message, stanza_id, message_id,
+            additional_data
+            FROM logs NATURAL JOIN jids WHERE jid IN ({jids})
+            AND account_id = {account_id}
             AND message_id = ?
-            '''
-        row = self._con.execute(
-            sql, (jid_id, account_id, correct_id)).fetchone()
-        if row is None:
-            return
+            AND time > ?
+            '''.format(jids=', '.join('?' * len(jids)),
+                       account_id=account_id)
+
+        return self._con.execute(
+            sql,
+            tuple(jids) + (message_id, min_time)).fetchone()
+
+    @timeit
+    def try_message_correction(self,
+                               account: str,
+                               jid: JID,
+                               nickname: Optional[str],
+                               corrected_text: str,
+                               correct_id: str,
+                               kind: KindConstant,
+                               timestamp: float) -> bool:
+
+        '''Try to correct a message
+
+        :param jid:  This can be a full jid or bare jid. A full jid only if the
+                     message is a MUC PM, otherwise a bare jid needs to be
+                     passed. `nickname` should only be passed if the message
+                     is a group chat message.
+        '''
+
+        account_id = self.get_account_id(account)
+        max_timestamp = timestamp - MAX_MESSAGE_CORRECTION_DELAY
+
+        self._log.debug(
+            'Check if message is correctable, parameters: %s %s %s %s %s',
+            jid, account_id, nickname, correct_id, max_timestamp)
+
+        sql = '''SELECT log_line_id, message, additional_data
+                 FROM logs
+                 NATURAL JOIN jids jid_id
+                 WHERE +jid = ?
+                 AND account_id = ?
+                 AND contact_name IS ?
+                 AND message_id = ?
+                 AND kind = ?
+                 AND time > ?
+                 '''
+
+        rows = self._con.execute(sql, (jid,
+                                       account_id,
+                                       nickname,
+                                       correct_id,
+                                       kind,
+                                       max_timestamp
+                                       )).fetchall()
+
+        if not rows:
+            self._log.debug('No correctable messages found')
+            return False
+
+        if len(rows) != 1:
+            self._log.warning('More than one correctable message found')
+            return False
+
+        row = rows[0]
 
         if row.additional_data is None:
             additional_data = AdditionalDataDict()
@@ -1037,6 +1090,8 @@ class MessageArchiveStorage(SqliteStorage):
         self._con.execute(
             sql, (corrected_text, serialized_dict, row.log_line_id))
 
+        return True
+
     @timeit
     def update_additional_data(self,
                                account: str,
