@@ -28,9 +28,8 @@ from gi.repository import Pango
 from gi.repository import GObject
 
 from gajim.common import app
-from gajim.common import ged
 from gajim.common import passwords
-from gajim.common.events import AccountDisonnected
+from gajim.common.const import ClientState
 from gajim.common.i18n import _
 from gajim.common.i18n import Q_
 from gajim.common.settings import AllSettingsT
@@ -541,29 +540,20 @@ class AccountRow(Gtk.ListBoxRow):
                           state: bool,
                           account: str
                           ) -> int:
-        def _on_disconnect(event: AccountDisonnected) -> None:
-            if event.account != account:
-                return
-            app.ged.remove_event_handler('account-disconnected',
-                                         ged.CORE,
-                                         _on_disconnect)
-            app.interface.disable_account(account)
 
         def _disable() -> None:
-            app.ged.register_event_handler('account-disconnected',
-                                           ged.CORE,
-                                           _on_disconnect)
             client = app.get_client(account)
+            client.connect_signal('state-changed', self._on_state_changed)
             client.change_status('offline', 'offline')
             switch.set_state(state)
             self._set_label(state)
 
-        old_state = app.settings.get_account_setting(account, 'active')
-        if old_state == state:
+        account_is_active = app.settings.get_account_setting(account, 'active')
+        if account_is_active == state:
             return Gdk.EVENT_PROPAGATE
 
-        if (account in app.connections and
-                not app.connections[account].state.is_disconnected):
+        if (account_is_active and
+                not app.get_client(account).state.is_disconnected):
             # Connecting or connected
             window = cast(AccountsWindow, get_app_window('AccountsWindow'))
             ConfirmationDialog(
@@ -585,6 +575,15 @@ class AccountRow(Gtk.ListBoxRow):
 
         return Gdk.EVENT_PROPAGATE
 
+    def _on_state_changed(self,
+                          client: types.Client,
+                          _signal_name: str,
+                          client_state: ClientState
+                          ) -> None:
+
+        if client_state.is_disconnected:
+            app.interface.disable_account(client.account)
+
 
 class AddNewAccountPage(Gtk.Box):
     def __init__(self) -> None:
@@ -872,8 +871,6 @@ class PrivacyPage(GenericSettingPage):
         app.settings.set_contact_settings('send_marker', None)
         app.settings.set_group_chat_settings(
             'send_marker', None, context='private')
-        for ctrl in app.window.get_controls(account=self._account):
-            ctrl.update_actions()
 
 
 class ConnectionPage(GenericSettingPage):
@@ -1005,7 +1002,7 @@ class PriorityDialog(SettingsDialog):
 
     def on_destroy(self, *args: Any) -> None:
         # Update priority
-        if self.account not in app.connections:
+        if self.account not in app.settings.get_active_accounts():
             return
         client = app.get_client(self.account)
         show = client.status
