@@ -26,6 +26,8 @@ from pathlib import Path
 
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
+from gi.repository import Pango
+from gi.repository import PangoCairo
 import cairo
 from nbxmpp.protocol import JID
 
@@ -38,8 +40,7 @@ from gajim.common.const import AvatarSize
 from gajim.common.const import StyleAttr
 
 from .const import DEFAULT_WORKSPACE_COLOR
-from .emoji_data import is_emoji
-from .emoji_data import get_emoji_font
+from .emoji_data_gtk import get_emoji_data
 from .util import load_icon_surface
 from .util import load_pixbuf
 from .util import text_to_color
@@ -52,8 +53,25 @@ from .util import make_rgba
 log = logging.getLogger('gajim.gui.avatar')
 
 
-AvatarCacheT = dict[JID, dict[tuple[int, int, Optional[str]],
-                              cairo.ImageSurface]]
+AvatarCacheT = dict[Union[JID, str], dict[tuple[int, int, Optional[str]],
+                                          cairo.ImageSurface]]
+
+
+def generate_avatar_letter(text: str) -> str:
+    if not text:
+        return ''
+
+    if text[0].isalpha():
+        return text[0].upper()
+
+    emoji_data = get_emoji_data()
+
+    # Max (arbitrary) length for emoji ZJW sequences: 11
+    for length in range(11, 0, -1):
+        if text[:length] in emoji_data.values():
+            return text[:length]
+
+    return text[0].upper()
 
 
 def generate_avatar(letters: str,
@@ -70,7 +88,7 @@ def generate_avatar(letters: str,
 
     width = size
     height = size
-    font_size = size * 0.5
+    font_size = size * 0.4
 
     # Set up surface
     surface = cairo.ImageSurface(cairo.Format.ARGB32, width, height)
@@ -81,28 +99,27 @@ def generate_avatar(letters: str,
     context.fill()
 
     # Draw letters
-    if is_emoji(letters):
-        font_face = get_emoji_font()
-    else:
-        font_face = 'sans-serif'
-    context.select_font_face(font_face,
-                             cairo.FontSlant.NORMAL,
-                             cairo.FontWeight.NORMAL)
-    context.set_font_size(font_size)
-    extends = context.text_extents(letters)
-    x_bearing = extends.x_bearing
-    y_bearing = extends.y_bearing
-    ex_width = extends.width
-    ex_height = extends.height
+    layout = PangoCairo.create_layout(context)
+    layout.set_text(generate_avatar_letter(letters))
+
+    description = Pango.FontDescription()
+    description.set_family('Sans')
+    description.set_size(int(font_size * Pango.SCALE))
+
+    layout.set_font_description(description)
+
+    _ink_rect, logical_rect = layout.get_extents()
+    layout_width = logical_rect.width / Pango.SCALE
+    layout_height = logical_rect.height / Pango.SCALE
+    x_pos = (width - layout_width) / 2
+    y_pos = (height - layout_height) / 2
 
-    x_pos = width / 2 - (ex_width / 2 + x_bearing)
-    y_pos = height / 2 - (ex_height / 2 + y_bearing)
     context.move_to(x_pos, y_pos)
     context.set_source_rgb(0.95, 0.95, 0.95)
     context.set_operator(cairo.Operator.OVER)
-    context.show_text(letters)
+    PangoCairo.show_layout(context, layout)
 
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 @lru_cache(maxsize=None)
@@ -132,7 +149,8 @@ def make_workspace_avatar(letter: str,
 
 
 def add_status_to_avatar(surface: cairo.ImageSurface,
-                         show: str) -> cairo.ImageSurface:
+                         show: str
+                         ) -> cairo.ImageSurface:
 
     width = surface.get_width()
     height = surface.get_height()
@@ -179,11 +197,15 @@ def add_status_to_avatar(surface: cairo.ImageSurface,
         context.set_line_width(clip_radius / 4)
         context.stroke()
 
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 @lru_cache(maxsize=128)
-def get_show_circle(show, size: int, scale: int) -> cairo.ImageSurface:
+def get_show_circle(show: Union[str, types.PresenceShowT],
+                    size: int,
+                    scale: int
+                    ) -> cairo.ImageSurface:
+
     if not isinstance(show, str):
         show = show.value
 
@@ -212,7 +234,7 @@ def get_show_circle(show, size: int, scale: int) -> cairo.ImageSurface:
         context.set_line_width(size / 10)
         context.stroke()
 
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 def fit(surface: cairo.ImageSurface, size: int) -> cairo.ImageSurface:
@@ -235,7 +257,7 @@ def fit(surface: cairo.ImageSurface, size: int) -> cairo.ImageSurface:
     context.scale(factor, factor)
     context.set_source_surface(square_surface, 0, 0)
     context.paint()
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 def square(surface: cairo.ImageSurface, size: int) -> cairo.ImageSurface:
@@ -259,7 +281,7 @@ def square(surface: cairo.ImageSurface, size: int) -> cairo.ImageSurface:
 
     context.set_source_surface(surface, x_pos, y_pos)
     context.paint()
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 def clip(surface: cairo.ImageSurface, mode: str) -> cairo.ImageSurface:
@@ -289,7 +311,7 @@ def clip_circle(surface: cairo.ImageSurface) -> cairo.ImageSurface:
     context.clip()
     context.paint()
 
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 def round_corners(surface: cairo.ImageSurface) -> cairo.ImageSurface:
@@ -320,7 +342,7 @@ def round_corners(surface: cairo.ImageSurface) -> cairo.ImageSurface:
 
     context.paint()
 
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 def convert_to_greyscale(surface: cairo.ImageSurface) -> cairo.ImageSurface:
@@ -333,14 +355,14 @@ def convert_to_greyscale(surface: cairo.ImageSurface) -> cairo.ImageSurface:
     context.set_source_rgba(1, 1, 1, 0.5)
     context.rectangle(0, 0, surface.get_width(), surface.get_height())
     context.fill()
-    return context.get_target()
+    return context.get_target()  # pyright: ignore
 
 
 class AvatarStorage(metaclass=Singleton):
     def __init__(self):
         self._cache: AvatarCacheT = defaultdict(dict)
 
-    def invalidate_cache(self, jid: JID) -> None:
+    def invalidate_cache(self, jid: Union[JID, str]) -> None:
         self._cache.pop(jid, None)
 
     def get_pixbuf(self,
@@ -373,6 +395,7 @@ class AvatarStorage(metaclass=Singleton):
 
         if transport_icon is not None:
             surface = load_icon_surface(transport_icon, size, scale)
+            assert surface is not None
             if show is not None:
                 surface = add_status_to_avatar(surface, show)
             self._cache[jid][(size, scale, show)] = surface
@@ -393,7 +416,7 @@ class AvatarStorage(metaclass=Singleton):
         name = contact.name
         color_string = str(contact.jid)
 
-        letter = self._generate_letter(name)
+        letter = generate_avatar_letter(name)
         surface = generate_default_avatar(
             letter, color_string, size, scale, style=style)
         if show is not None:
@@ -412,11 +435,12 @@ class AvatarStorage(metaclass=Singleton):
 
         if transport_icon is not None:
             surface = load_icon_surface(transport_icon, size, scale)
-            self._cache[jid][(size, scale)] = surface
+            assert surface is not None
+            self._cache[jid][(size, scale, None)] = surface
             return surface
 
         if not default:
-            surface = self._cache[jid].get((size, scale))
+            surface = self._cache[jid].get((size, scale, None))
             if surface is not None:
                 return surface
 
@@ -425,18 +449,18 @@ class AvatarStorage(metaclass=Singleton):
                 surface = self.surface_from_filename(avatar_sha, size, scale)
                 if surface is not None:
                     surface = clip(surface, style)
-                    self._cache[jid][(size, scale)] = surface
+                    self._cache[jid][(size, scale, None)] = surface
                     return surface
 
                 # avatar_sha set, but image is missing
                 # (e.g. avatar cache deleted)
                 app.storage.cache.set_muc(jid, 'avatar', None)
 
-        con = app.connections[account]
-        name = get_groupchat_name(con, jid)
-        letter = self._generate_letter(name)
+        client = app.get_client(account)
+        name = get_groupchat_name(client, jid)
+        letter = generate_avatar_letter(name)
         surface = generate_default_avatar(letter, str(jid), size, scale, style)
-        self._cache[jid][(size, scale)] = surface
+        self._cache[jid][(size, scale, None)] = surface
         return surface
 
     def get_workspace_surface(self,
@@ -444,7 +468,7 @@ class AvatarStorage(metaclass=Singleton):
                               size: int,
                               scale: int) -> Optional[cairo.ImageSurface]:
 
-        surface = self._cache[workspace_id].get((size, scale))
+        surface = self._cache[workspace_id].get((size, scale, None))
         if surface is not None:
             return surface
 
@@ -462,10 +486,9 @@ class AvatarStorage(metaclass=Singleton):
             app.settings.set_workspace_setting(workspace_id, 'avatar_sha', '')
 
         rgba = make_rgba(color or DEFAULT_WORKSPACE_COLOR)
-        letter = name[:1].upper()
         surface = make_workspace_avatar(
-            letter, rgba_to_float(rgba), size, scale)
-        self._cache[workspace_id][(size, scale)] = surface
+            name, rgba_to_float(rgba), size, scale)
+        self._cache[workspace_id][(size, scale, None)] = surface
         return surface
 
     @staticmethod
@@ -483,6 +506,7 @@ class AvatarStorage(metaclass=Singleton):
                                          height,
                                          GdkPixbuf.InterpType.BILINEAR)
 
+        assert pixbuf is not None
         return pixbuf.save_to_bufferv('png', [], [])
 
     @staticmethod
@@ -563,10 +587,3 @@ class AvatarStorage(metaclass=Singleton):
         if surface is None:
             return None
         return clip(surface, style)
-
-    @staticmethod
-    def _generate_letter(name: str) -> str:
-        for letter in name:
-            if letter.isalpha():
-                return letter.capitalize()
-        return name[0].capitalize()
