@@ -171,7 +171,6 @@ APP_ACTIONS = [
     ('file-transfer', None),
     ('forget-groupchat', 'a{sv}'),
     ('groupchat-join', 'as'),
-    ('ipython', None),
     ('join-support-chat', None),
     ('manage-proxies', None),
     ('open-link', 'as'),
@@ -182,13 +181,86 @@ APP_ACTIONS = [
     ('remove-history', 'a{sv}'),
     ('shortcuts', None),
     ('show', None),
-    ('start-chat', 's'),
+    ('start-chat', 'as'),
     ('xml-console', None),
 ]
 
 
+MAIN_WIN_ACTIONS = [
+    # action name, variant type, enabled
+    ('input-bold', None, False),
+    ('input-italic', None, False),
+    ('input-strike', None, False),
+    ('input-clear', None, False),
+    ('show-emoji-chooser', None, False),
+    ('correct-message', None, False),
+    ('quote', 's', False),
+    ('mention', 's', False),
+    ('send-file-httpupload', None, False),
+    ('send-file-jingle', None, False),
+    ('send-file', None, False),
+    ('invite-contacts', None, False),
+    ('add-to-roster', None, True),
+    ('start-voice-call', None, False),
+    ('start-video-call', None, False),
+    ('show-contact-info', None, False),
+    ('send-message', None, False),
+    ('muc-change-nickname', None, False),
+    ('muc-invite', None, False),
+    ('muc-contact-info', 's', False),
+    ('muc-execute-command', 's', False),
+    ('muc-ban', 's', False),
+    ('muc-kick', 's', False),
+    ('muc-change-role', 'as', False),
+    ('muc-change-affiliation', 'as', False),
+    ('muc-request-voice', None, False),
+    ('scroll-view-up', None, True),
+    ('scroll-view-down', None, True),
+    ('change-nickname', None, True),
+    ('change-subject', None, True),
+    ('escape', None, True),
+    ('close-chat', None, True),
+    ('restore-chat', None, True),
+    ('switch-next-chat', None, True),
+    ('switch-prev-chat', None, True),
+    ('switch-next-unread-chat', None, True),
+    ('switch-prev-unread-chat', None, True),
+    ('switch-chat-1', None, True),
+    ('switch-chat-2', None, True),
+    ('switch-chat-3', None, True),
+    ('switch-chat-4', None, True),
+    ('switch-chat-5', None, True),
+    ('switch-chat-6', None, True),
+    ('switch-chat-7', None, True),
+    ('switch-chat-8', None, True),
+    ('switch-chat-9', None, True),
+    ('switch-workspace-1', None, True),
+    ('switch-workspace-2', None, True),
+    ('switch-workspace-3', None, True),
+    ('switch-workspace-4', None, True),
+    ('switch-workspace-5', None, True),
+    ('switch-workspace-6', None, True),
+    ('switch-workspace-7', None, True),
+    ('switch-workspace-8', None, True),
+    ('switch-workspace-9', None, True),
+    ('toggle-chat-list', None, True),
+    ('add-workspace', 's', True),
+    ('edit-workspace', 's', True),
+    ('remove-workspace', 's', True),
+    ('activate-workspace', 's', True),
+    ('add-chat', 'a{sv}', True),
+    ('add-group-chat', 'as', True),
+    ('add-to-roster', 'a{sv}', True),
+]
+
+
 ACCOUNT_ACTIONS = [
     ('add-contact', 'as'),
+    ('contact-info', 's'),
+    ('block-contact', 's'),
+    ('remove-contact', 's'),
+    ('execute-command', 's'),
+    ('modify-gateway', 's'),
     ('archive', 's'),
     ('blocking', 's'),
     ('bookmarks', 's'),
@@ -214,6 +286,10 @@ ALWAYS_ACCOUNT_ACTIONS = {
 
 ONLINE_ACCOUNT_ACTIONS = {
     'add-contact',
+    'remove-contact',
+    'contact-info',
+    'execute-command',
+    'modify-gateway',
     'bookmarks',
     'import-contacts',
     'open-chat',
@@ -222,10 +298,12 @@ ONLINE_ACCOUNT_ACTIONS = {
     'server-info',
     'services',
     'sync-history',
+    'mark-as-read',
 }
 
 
 FEATURE_ACCOUNT_ACTIONS = {
     'archive',
     'blocking',
+    'block-contact',
 }
