@@ -15,6 +15,7 @@
 from __future__ import annotations
 
 from typing import Optional
+from typing import Union
 
 from datetime import datetime
 from datetime import timedelta
@@ -24,13 +25,13 @@ from gi.repository import Gdk
 from gi.repository import GdkPixbuf
 from gi.repository import GLib
 from gi.repository import Gtk
-from gi.repository import GObject
 from gi.repository import Pango
 
 import cairo
 
 from nbxmpp.errors import StanzaError
 from nbxmpp.modules.security_labels import Displaymarking
+from nbxmpp.structs import CommonError
 
 from gajim.common import app
 from gajim.common.const import AvatarSize
@@ -39,10 +40,10 @@ from gajim.common.const import TRUST_SYMBOL_DATA
 from gajim.common.helpers import AdditionalDataDict
 from gajim.common.helpers import from_one_line
 from gajim.common.helpers import get_group_chat_nick
-from gajim.common.helpers import get_muc_context
 from gajim.common.helpers import message_needs_highlight
 from gajim.common.helpers import to_user_string
 from gajim.common.i18n import _
+from gajim.common.i18n import is_rtl_text
 from gajim.common.i18n import Q_
 from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.types import ChatContactT
@@ -52,7 +53,9 @@ from .widgets import MoreMenuButton
 from ..message_widget import MessageWidget
 from ...dialogs import InputDialog
 from ...dialogs import DialogButton
+from ...menus import get_groupchat_participant_menu
 from ...preview import PreviewWidget
+from ...util import GajimPopover
 from ...util import format_fingerprint
 from ...util import get_cursor
 
@@ -60,20 +63,6 @@ MERGE_TIMEFRAME = timedelta(seconds=120)
 
 
 class MessageRow(BaseRow):
-
-    __gsignals__ = {
-        'mention': (
-            GObject.SignalFlags.RUN_LAST,
-            None,
-            (str,)
-        ),
-        'quote': (
-            GObject.SignalFlags.RUN_LAST,
-            None,
-            (str,)
-        ),
-    }
-
     def __init__(self,
                  account: str,
                  contact: ChatContactT,
@@ -86,8 +75,7 @@ class MessageRow(BaseRow):
                  additional_data: Optional[AdditionalDataDict] = None,
                  display_marking: Optional[Displaymarking] = None,
                  marker: Optional[str] = None,
-                 error: Optional[StanzaError] = None,
-                 encryption_enabled: bool = False,
+                 error: Union[CommonError, StanzaError, None] = None,
                  log_line_id: Optional[int] = None) -> None:
 
         BaseRow.__init__(self, account)
@@ -129,12 +117,12 @@ class MessageRow(BaseRow):
                 text, additional_data)
 
         if is_previewable:
-            context = None
-            if self._is_groupchat:
-                context = get_muc_context(self._contact.jid)
+            muc_context = None
+            if isinstance(self._contact, GroupchatContact):
+                muc_context = self._contact.muc_context
             self._message_widget = PreviewWidget(account)
             app.preview_manager.create_preview(
-                text, self._message_widget, from_us, context)
+                text, self._message_widget, from_us, muc_context)
         else:
             self._message_widget = MessageWidget(account)
             self._message_widget.add_with_styling(text, nickname=name)
@@ -155,28 +143,12 @@ class MessageRow(BaseRow):
         timestamp_label = self.create_timestamp_widget(self.timestamp)
         self._meta_box.pack_start(timestamp_label, False, True, 0)
 
-        if kind in ('incoming', 'incoming_queue', 'outgoing'):
-            if additional_data is not None:
-                encryption_img = self._get_encryption_image(
-                    additional_data, encryption_enabled)
-                if encryption_img:
-                    self._meta_box.pack_start(encryption_img, False, True, 0)
+        if additional_data is not None:
+            encryption_img = self._get_encryption_image(additional_data)
+            if encryption_img:
+                self._meta_box.pack_start(encryption_img, False, True, 0)
 
-        if display_marking and app.settings.get_account_setting(
-                account, 'enable_security_labels'):
-            label_text = GLib.markup_escape_text(display_marking.name)
-            if label_text:
-                display_marking_label = Gtk.Label()
-                display_marking_label.set_ellipsize(Pango.EllipsizeMode.END)
-                display_marking_label.set_max_width_chars(30)
-                display_marking_label.set_tooltip_text(label_text)
-                bgcolor = display_marking.bgcolor
-                fgcolor = display_marking.fgcolor
-                label_text = (
-                    f'<span size="small" bgcolor="{bgcolor}" '
-                    f'fgcolor="{fgcolor}"><tt>{label_text}</tt></span>')
-                display_marking_label.set_markup(label_text)
-                self._meta_box.add(display_marking_label)
+        self._add_security_label(display_marking)
 
         self._message_icons = MessageIcons()
 
@@ -222,6 +194,10 @@ class MessageRow(BaseRow):
         self._bottom_box = Gtk.Box(spacing=6)
         self._bottom_box.add(self._message_widget)
 
+        if is_rtl_text(text):
+            self._bottom_box.set_halign(Gtk.Align.END)
+            self._message_widget.set_direction(Gtk.TextDirection.RTL)
+
         more_menu_button = MoreMenuButton(self, self._contact, name)
         more_menu_button.set_hexpand(True)
         more_menu_button.set_halign(Gtk.Align.END)
@@ -233,6 +209,31 @@ class MessageRow(BaseRow):
 
         self.show_all()
 
+    def _add_security_label(self,
+                            display_marking: Optional[Displaymarking]
+                            ) -> None:
+
+        if display_marking is None:
+            return
+
+        if not app.settings.get_account_setting(self._account,
+                                                'enable_security_labels'):
+            return
+
+        label_text = GLib.markup_escape_text(display_marking.name)
+        if label_text:
+            display_marking_label = Gtk.Label()
+            display_marking_label.set_ellipsize(Pango.EllipsizeMode.END)
+            display_marking_label.set_max_width_chars(30)
+            display_marking_label.set_tooltip_text(label_text)
+            bgcolor = display_marking.bgcolor
+            fgcolor = display_marking.fgcolor
+            label_text = (
+                f'<span size="small" bgcolor="{bgcolor}" '
+                f'fgcolor="{fgcolor}"><tt>{label_text}</tt></span>')
+            display_marking_label.set_markup(label_text)
+            self._meta_box.add(display_marking_label)
+
     def _check_for_highlight(self, text: str) -> None:
         assert isinstance(self._contact, GroupchatContact)
         if self._contact.nickname is None:
@@ -270,10 +271,34 @@ class MessageRow(BaseRow):
                            event: Gdk.EventButton,
                            name: str
                            ) -> int:
-        if event.type == Gdk.EventType.BUTTON_PRESS and event.button == 1:
-            self.emit('mention', name)
+        if event.type == Gdk.EventType.BUTTON_PRESS:
+            if event.button == 1:
+                app.window.activate_action('mention', GLib.Variant('s', name))
+            elif event.button == 3:
+                self._show_participant_menu(name, event)
+
         return Gdk.EVENT_STOP
 
+    def _show_participant_menu(self, nick: str, event: Gdk.EventButton) -> None:
+        assert isinstance(self._contact, GroupchatContact)
+        if not self._contact.is_joined:
+            return
+
+        self_contact = self._contact.get_self()
+        assert self_contact is not None
+
+        if nick == self_contact.name:
+            # Don’t show menu for us
+            return
+
+        contact = self._contact.get_resource(nick)
+        menu = get_groupchat_participant_menu(self._contact.account,
+                                              self_contact,
+                                              contact)
+
+        popover = GajimPopover(menu, relative_to=self, event=event)
+        popover.popup()
+
     @staticmethod
     def _on_realize(event_box: Gtk.EventBox) -> None:
         window = event_box.get_window()
@@ -333,7 +358,11 @@ class MessageRow(BaseRow):
         clip.set_text(f'{timestamp_formatted} - {self.name}: {text}', -1)
 
     def on_quote_message(self, _widget: Gtk.Widget) -> None:
-        self.emit('quote', self._message_widget.get_text())
+        app.window.activate_action(
+            'quote', GLib.Variant('s', self._message_widget.get_text()))
+
+    def on_correct_message(self, _widget: Gtk.Widget) -> None:
+        app.window.activate_action('correct-message', None)
 
     def on_retract_message(self, _widget: Gtk.Widget) -> None:
         def _on_retract(reason: str) -> None:
@@ -353,12 +382,12 @@ class MessageRow(BaseRow):
 
     def _get_encryption_image(self,
                               additional_data: AdditionalDataDict,
-                              encryption_enabled: bool = False
                               ) -> Optional[Gtk.Image]:
+
         details = self._get_encryption_details(additional_data)
         if details is None:
             # Message was not encrypted
-            if not encryption_enabled:
+            if not self._contact.settings.get('encryption'):
                 return None
             icon = 'channel-insecure-symbolic'
             color = 'unencrypted-color'
@@ -418,6 +447,13 @@ class MessageRow(BaseRow):
             self._message_widget.destroy()
             self._message_widget = MessageWidget(self._account)
             self._bottom_box.pack_start(self._message_widget, True, True, 0)
+            if is_rtl_text(text):
+                self._bottom_box.set_halign(Gtk.Align.END)
+                self._message_widget.set_direction(Gtk.TextDirection.RTL)
+            else:
+                self._bottom_box.set_halign(Gtk.Align.FILL)
+                self._message_widget.set_direction(Gtk.TextDirection.LTR)
+
         self._message_widget.add_with_styling(text)
         self.get_style_context().add_class('retracted-message')
 
@@ -425,6 +461,13 @@ class MessageRow(BaseRow):
         if not isinstance(self._message_widget, PreviewWidget):
             self._message_widget.add_with_styling(text, nickname)
 
+            if is_rtl_text(text):
+                self._bottom_box.set_halign(Gtk.Align.END)
+                self._message_widget.set_direction(Gtk.TextDirection.RTL)
+            else:
+                self._bottom_box.set_halign(Gtk.Align.FILL)
+                self._message_widget.set_direction(Gtk.TextDirection.LTR)
+
         self._has_receipt = False
         self._message_icons.set_receipt_icon_visible(False)
         self._message_icons.set_correction_icon_visible(True)
