@@ -29,7 +29,7 @@ from gajim.common import ged
 from gajim.common.const import MUC_CREATION_EXAMPLES
 from gajim.common.const import MUC_DISCO_ERRORS
 from gajim.common.events import AccountConnected
-from gajim.common.events import AccountDisonnected
+from gajim.common.events import AccountDisconnected
 from gajim.common.ged import EventHelper
 from gajim.common.helpers import get_random_muc_localpart
 from gajim.common.helpers import validate_jid
@@ -85,7 +85,7 @@ class CreateGroupchatWindow(Gtk.ApplicationWindow, EventHelper):
         self.set_focus(self._ui.address_entry)
 
     def _on_account_state(self,
-                          _event: Union[AccountConnected, AccountDisonnected]
+                          _event: Union[AccountConnected, AccountDisconnected]
                           ) -> None:
         any_account_connected = app.get_number_of_connected_accounts() > 0
         if any_account_connected:
@@ -231,6 +231,10 @@ class CreateGroupchatWindow(Gtk.ApplicationWindow, EventHelper):
 
     def _on_public_private_toggled(self, _radiobutton: Gtk.RadioButton) -> None:
         is_public = self._ui.public_radio.get_active()
+        if is_public:
+            self._validate_jid(self._ui.address_entry.get_text())
+        else:
+            self._ui.create_button.set_sensitive(True)
         self._ui.address_label.set_visible(is_public)
         self._ui.address_entry.set_visible(is_public)
 
