@@ -40,7 +40,6 @@ from gajim.common.setting_values import AllSettings
 from .const import Setting
 from .const import SettingKind
 from .const import SettingType
-from .controls.groupchat import GroupchatControl
 from .dialogs import DialogButton
 from .dialogs import ConfirmationDialog
 from .emoji_chooser import emoji_chooser
@@ -49,12 +48,13 @@ from .settings import SettingsBox
 from .settings import SettingsDialog
 from .sidebar_switcher import SideBarSwitcher
 from .video_preview import VideoPreview
+from .preview import PREVIEW_CLICK_ACTIONS
 from .util import open_window
 from .util import get_app_window
 from .builder import get_builder
 
-if app.is_installed('GSPELL'):
-    from gi.repository import Gspell  # pylint: disable=ungrouped-imports
+# if app.is_installed('GSPELL'):
+#     from gi.repository import Gspell  # pylint: disable=ungrouped-imports
 
 log = logging.getLogger('gajim.gui.preferences')
 
@@ -272,8 +272,7 @@ class Chats(PreferenceBox):
                     SettingType.CONFIG,
                     'use_speller',
                     desc=speller_desc,
-                    enabled_func=self._speller_available,
-                    callback=self._on_use_speller),
+                    enabled_func=self._speller_available),
 
             Setting(SettingKind.SWITCH,
                     _('Message Receipts (✔)'),
@@ -318,21 +317,6 @@ class Chats(PreferenceBox):
     def _speller_available() -> bool:
         return app.is_installed('GSPELL')
 
-    @staticmethod
-    def _on_use_speller(value: bool, *args: Any) -> None:
-        if not value:
-            return
-
-        lang = app.settings.get('speller_language')
-        gspell_lang = Gspell.language_lookup(lang)
-        if gspell_lang is None:
-            gspell_lang = Gspell.language_get_default()
-        if gspell_lang is None:
-            return
-        app.settings.set('speller_language', gspell_lang.get_code())
-        for ctrl in app.window.get_controls():
-            ctrl.set_speller()
-
     @staticmethod
     def _on_show_send_message_button(show_button: bool, *args: Any) -> None:
         # Bind to send_on_ctrl_enter to make this setting user-friendly
@@ -401,19 +385,13 @@ class GroupChats(PreferenceBox):
 
     @staticmethod
     def _on_sort_by_show_in_muc(_value: bool, *args: Any) -> None:
-        for ctrl in app.window.get_controls():
-            if ctrl.is_groupchat:
-                assert isinstance(ctrl, GroupchatControl)
-                if ctrl.roster is not None:
-                    ctrl.roster.invalidate_sort()
+        roster = app.window.get_control().get_group_chat_roster()
+        roster.invalidate_sort()
 
     @staticmethod
     def _on_show_status_in_roster(_value: bool, *args: Any) -> None:
-        for ctrl in app.window.get_controls():
-            if ctrl.is_groupchat:
-                assert isinstance(ctrl, GroupchatControl)
-                if ctrl.roster is not None:
-                    ctrl.roster.draw_contacts()
+        roster = app.window.get_control().get_group_chat_roster()
+        roster.draw_contacts()
 
     @staticmethod
     def _reset_join_left(button: Gtk.Button) -> None:
@@ -436,14 +414,6 @@ class FilePreview(PreferenceBox):
             10485760: '10 MiB',
         }
 
-        actions = {
-            'open': _('Open'),
-            'save_as': _('Save As…'),
-            'open_folder': _('Open Folder'),
-            'copy_link_location': _('Copy Link Location'),
-            'open_link_in_browser': _('Open Link in Browser'),
-        }
-
         settings = [
             Setting(SettingKind.SWITCH,
                     _('File Preview'),
@@ -488,7 +458,7 @@ class FilePreview(PreferenceBox):
                     'preview_leftclick_action',
                     desc=_('Action when left-clicking a preview'),
                     bind='enable_file_preview',
-                    props={'entries': actions}),
+                    props={'entries': PREVIEW_CLICK_ACTIONS}),
 
             Setting(SettingKind.SWITCH,
                     _('HTTPS Verification'),
@@ -755,8 +725,10 @@ class Emoji(PreferenceBox):
 
     @staticmethod
     def _toggle_emoticons() -> None:
-        for ctrl in app.window.get_controls():
-            ctrl.toggle_emoticons()
+        # TODO
+        # for ctrl in app.window.get_controls():
+        #     ctrl.toggle_emoticons()
+        pass
 
 
 class Server(PreferenceBox):
