@@ -56,7 +56,7 @@ from .chat_filter import ChatFilter
 from .groupchat_info import GroupChatInfoScrolled
 from .groupchat_nick import NickChooser
 from .builder import get_builder
-from .util import get_icon_name
+from .util import GajimPopover, get_icon_name
 from .util import AccountBadge
 
 ContactT = Union[BareContact, GroupchatContact]
@@ -68,7 +68,11 @@ class Search(IntEnum):
 
 
 class StartChatDialog(Gtk.ApplicationWindow):
-    def __init__(self, jid: Optional[str] = None) -> None:
+    def __init__(self,
+                 jid: Optional[str] = None,
+                 initial_message: Optional[str] = None
+                 ) -> None:
+
         Gtk.ApplicationWindow.__init__(self)
         self.set_name('StartChatDialog')
         self.set_application(app.app)
@@ -138,16 +142,15 @@ class StartChatDialog(Gtk.ApplicationWindow):
         if rows:
             self._load_contacts(rows)
 
+        self._initial_message: dict[str, Optional[str]] = {}
         if jid is not None:
-            self.set_search_text(jid)
+            self._initial_message[jid] = initial_message
+            self._ui.search_entry.set_text(jid)
 
         self.select_first_row()
         self._ui.connect_signals(self)
         self.show_all()
 
-    def set_search_text(self, text: str) -> None:
-        self._ui.search_entry.set_text(text)
-
     def remove_row(self, account: str, jid: str) -> None:
         for row in cast(list[ContactRow], self._ui.listbox.get_children()):
             if row.account == account and row.jid == jid:
@@ -362,12 +365,14 @@ class StartChatDialog(Gtk.ApplicationWindow):
             self._disco_muc(row.account, jid, request_vcard=row.is_new)
 
         else:
+            initial_message = self._initial_message.get(str(jid))
             app.window.add_chat(
                 row.account,
                 jid,
                 'contact',
                 select=True,
-                workspace='current')
+                workspace='current',
+                message=initial_message)
             self.ready_to_destroy = True
             self.destroy()
 
@@ -428,13 +433,14 @@ class StartChatDialog(Gtk.ApplicationWindow):
             jid,
             request_vcard=request_vcard,
             allow_redirect=True,
+            timeout=10,
             callback=self._muc_disco_info_received,
             user_data=account)
 
     def _muc_disco_info_received(self, task: Task) -> None:
         try:
             result = cast(DiscoInfo, task.finish())
-        except StanzaError as error:
+        except (StanzaError, TimeoutStanzaError) as error:
             self._set_error(error)
             return
 
@@ -450,12 +456,15 @@ class StartChatDialog(Gtk.ApplicationWindow):
         else:
             self._set_error_from_code('not-muc-service')
 
-    def _set_error(self, error: StanzaError) -> None:
-        text = MUC_DISCO_ERRORS.get(error.condition, to_user_string(error))
-        if error.condition == 'gone':
-            reason = error.get_text(get_rfc5646_lang())
-            if reason:
-                text = f'{text}:\n{reason}'
+    def _set_error(self, error: Union[StanzaError, TimeoutStanzaError]) -> None:
+        if isinstance(error, TimeoutStanzaError):
+            text = _('This address is not reachable.')
+        else:
+            text = MUC_DISCO_ERRORS.get(error.condition, to_user_string(error))
+            if error.condition == 'gone':
+                reason = error.get_text(get_rfc5646_lang())
+                if reason:
+                    text = f'{text}:\n{reason}'
         self._show_error_page(text)
 
     def _set_error_from_code(self, error_code: str) -> None:
@@ -651,7 +660,7 @@ class StartChatDialog(Gtk.ApplicationWindow):
         accounts = app.get_connected_accounts()
         if not accounts:
             return
-        client = app.connections[accounts[0]].connection
+        client = app.get_client(accounts[0]).connection
 
         text = self._ui.search_entry.get_text().strip()
         self._global_search_listbox.start_search()
@@ -820,15 +829,7 @@ class ContactRow(Gtk.ListBoxRow):
 
         menu = get_start_chat_row_menu(self.account, self.jid)
 
-        rectangle = Gdk.Rectangle()
-        rectangle.x = int(event.x)
-        rectangle.y = int(event.y)
-        rectangle.width = rectangle.height = 1
-
-        popover = Gtk.Popover.new_from_model(self, menu)
-        popover.set_relative_to(self)
-        popover.set_position(Gtk.PositionType.RIGHT)
-        popover.set_pointing_to(rectangle)
+        popover = GajimPopover(menu, relative_to=self, event=event)
         popover.popup()
 
     def _get_avatar_image(self, contact: ContactT) -> Gtk.Image:
