@@ -38,13 +38,11 @@ from typing import Dict
 from typing import Optional
 from typing import Union
 
-import os
 import sys
 import time
 import logging
 from functools import partial
 from threading import Thread
-from importlib.util import find_spec
 
 from gi.repository import Gtk
 from gi.repository import GLib
@@ -67,22 +65,26 @@ from gajim.common.events import FileCompleted
 from gajim.common.events import FileHashError
 from gajim.common.events import FileProgress
 from gajim.common.events import FileError
+from gajim.common.modules.contacts import BareContact
+from gajim.common.modules.contacts import GroupchatContact
 from gajim.common.structs import OutgoingMessage
 from gajim.common.i18n import _
 from gajim.common.client import Client
 from gajim.common.const import FTState
 from gajim.common.file_props import FileProp
+from gajim.common.types import ChatContactT
 
 from gajim.common.modules.httpupload import HTTPFileTransfer
 
 from gajim.gui.dialogs import ErrorDialog
 from gajim.gui.filechoosers import FileChooserDialog
+from gajim.gui.file_transfer_send import SendFileDialog
 from gajim.gui.filetransfer import FileTransfersWindow
 from gajim.gui.menus import build_accounts_menu
 from gajim.gui.util import get_app_window
 from gajim.gui.util import get_app_windows
 from gajim.gui.util import get_color_for_account
-from gajim.gui.types import ControlT
+from gajim.gui.control import ChatControl
 from gajim.plugins.repository import PluginRepository
 
 log = logging.getLogger('gajim.interface')
@@ -112,7 +114,7 @@ class Interface:
 
         self.instances: dict[str, Any] = {}
 
-        for acc in app.connections:
+        for acc in app.settings.get_active_accounts():
             self.instances[acc] = {
                 'infos': {},
                 'disco': {}
@@ -125,7 +127,6 @@ class Interface:
         # pylint: disable=line-too-long
         self.handlers = {
             'signed-in': [self.handle_event_signed_in],
-            'presence-received': [self.handle_event_presence],
             'message-sent': [self.handle_event_msgsent],
             'message-not-sent': [self.handle_event_msgnotsent],
         }
@@ -166,14 +167,6 @@ class Interface:
         if app.settings.get('ask_online_status'):
             app.window.show_account_page(account)
 
-    def handle_event_presence(self, event):
-        account = event.conn.name
-        jid = event.jid
-
-        ctrl = app.window.get_control(account, jid)
-        if ctrl and ctrl.session and len(event.contact_list) > 1:
-            ctrl.remove_session(ctrl.session)
-
     @staticmethod
     def handle_event_msgsent(event):
         if not event.play_sound:
@@ -310,7 +303,7 @@ class Interface:
             session.end_session()
 
     def send_httpupload(self,
-                        chat_control: ControlT,
+                        chat_control: ChatControl,
                         path: Optional[str] = None
                         ) -> None:
         if path is not None:
@@ -323,18 +316,19 @@ class Interface:
                           transient_for=app.window)
 
     def _on_file_dialog_ok(self,
-                           chat_control: ControlT,
+                           chat_control: ChatControl,
                            paths: list[str]
                            ) -> None:
         for path in paths:
             self._send_httpupload(chat_control, path)
 
-    def _send_httpupload(self, chat_control: ControlT, path: str) -> None:
-        client = app.get_client(chat_control.account)
+    def _send_httpupload(self, chat_control: ChatControl, path: str) -> None:
+        client = app.get_client(chat_control.contact.account)
+        encryption = chat_control.contact.settings.get('encryption') or None
         try:
             transfer = client.get_module('HTTPUpload').make_transfer(
                 path,
-                chat_control.encryption,
+                encryption,
                 chat_control.contact,
                 chat_control.is_groupchat)
         except exceptions.FileError as error:
@@ -377,6 +371,61 @@ class Interface:
         client = app.get_client(transfer.account)
         client.get_module('HTTPUpload').cancel_transfer(transfer)
 
+    def _get_pref_ft_method(self, contact: ChatContactT) -> Optional[str]:
+        ft_pref = app.settings.get_account_setting(contact.account,
+                                                   'filetransfer_preference')
+        httpupload = app.window.get_action_enabled('send-file-httpupload')
+        jingle = app.window.get_action_enabled('send-file-jingle')
+
+        if isinstance(contact, GroupchatContact):
+            if httpupload:
+                return 'httpupload'
+            return None
+
+        if httpupload and jingle:
+            return ft_pref
+
+        if httpupload:
+            return 'httpupload'
+
+        if jingle:
+            return 'jingle'
+        return None
+
+    def start_file_transfer(self,
+                            contact: ChatContactT,
+                            path: Optional[str] = None,
+                            method: Optional[str] = None
+                            ) -> None:
+
+        if method is None:
+            method = self._get_pref_ft_method(contact)
+            if method is None:
+                return
+
+        control = app.window.get_control()
+        if not control.has_active_chat():
+            return
+
+        if path is None:
+            if method == 'httpupload':
+                self.send_httpupload(control)
+                return
+            if method == 'jingle':
+                self.instances['file_transfers'].show_file_send_request(
+                    contact.account, contact)
+            return
+
+        if method == 'httpupload':
+            self.send_httpupload(control, path)
+        else:
+            assert isinstance(contact, BareContact)
+            send_callback = partial(
+                self.instances['file_transfers'].send_file,
+                contact.account,
+                contact)
+            SendFileDialog(contact, send_callback, app.window, [path])
+
     @staticmethod
     def create_groupchat(account: str,
                          room_jid: str,
@@ -408,18 +457,17 @@ class Interface:
                             jid: str,
                             message: Optional[str] = None
                             ) -> None:
+
         jid_ = JID.from_string(jid)
         if app.window.chat_exists(account, jid_):
             app.window.select_chat(account, jid_)
             if message is not None:
-                control = app.window.get_control(account, jid_)
-                if control is None:
-                    return
-                control.msg_textview.insert_text(message)
+                message_input = app.window.get_chat_stack().get_message_input()
+                message_input.insert_text(message)
             return
 
-        # TODO: handle message arg in StartChat
-        app.app.activate_action('start-chat', GLib.Variant('s', str(jid)))
+        app.app.activate_action(
+            'start-chat', GLib.Variant('as', [str(jid), message or '']))
 
     @staticmethod
     def create_account(account: str,
@@ -497,7 +545,7 @@ class Interface:
 
         app.ged.raise_event(AccountEnabled(account=account))
 
-        app.connections[account].change_status('online', '')
+        app.get_client(account).change_status('online', '')
         window = get_app_window('AccountsWindow')
         if window is not None:
             GLib.idle_add(window.enable_account, account, True)
@@ -515,15 +563,15 @@ class Interface:
         build_accounts_menu()
         app.app.update_app_actions_state()
 
-        app.ged.raise_event(AccountDisabled(account=account))
-
-        app.connections[account].cleanup()
+        app.get_client(account).cleanup()
         del app.connections[account]
         del self.instances[account]
         del app.nicks[account]
         del app.automatic_rooms[account]
         del app.to_be_removed[account]
 
+        app.ged.raise_event(AccountDisabled(account=account))
+
     def remove_account(self, account: str) -> None:
         if app.settings.get_account_setting(account, 'active'):
             self.disable_account(account)
@@ -544,8 +592,10 @@ class Interface:
         Auto connect at startup
         '''
 
-        for account, con in app.connections.items():
-            if not app.settings.get_account_setting(account, 'autoconnect'):
+        for client in app.get_clients():
+            account = client.account
+            if not app.settings.get_account_setting(account,
+                                                    'autoconnect'):
                 continue
 
             status = 'online'
@@ -558,7 +608,7 @@ class Interface:
                     account, 'last_status_msg')
                 status_message = helpers.from_one_line(status_message)
 
-            con.change_status(status, status_message)
+            client.change_status(status, status_message)
 
     def change_status(self,
                       status: str,
@@ -572,12 +622,12 @@ class Interface:
             self._change_status(account, status)
             return
 
-        for acc in app.connections:
-            if not app.settings.get_account_setting(acc,
+        for client in app.get_clients():
+            if not app.settings.get_account_setting(client.account,
                                                     'sync_with_global_status'):
                 continue
 
-            self._change_status(acc, status)
+            self._change_status(client.account, status)
 
     @staticmethod
     def _change_status(account: str, status: str) -> None:
@@ -628,47 +678,6 @@ class Interface:
     def avatar_exists(self, filename: str) -> bool:
         return app.app.avatar_storage.get_avatar_path(filename) is not None
 
-    @staticmethod
-    def create_ipython_window() -> None:
-        # Check if IPython is installed
-        ipython = find_spec('IPython')
-        is_installed = ipython is not None
-        if not is_installed:
-            # Abort early to avoid tracebacks
-            print('IPython is not installed')
-            return
-        try:
-            from gajim.dev.ipython_view import IPythonView
-        except ImportError:
-            print('ipython_view not found')
-            return
-        from gi.repository import Pango
-
-        if os.name == 'nt':
-            font = 'Lucida Console 9'
-        else:
-            font = 'Luxi Mono 10'
-
-        window = Gtk.Window()
-        window.set_title(_('Gajim: IPython Console'))
-        window.set_size_request(750, 550)
-        window.set_resizable(True)
-        sw = Gtk.ScrolledWindow()
-        sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
-        view = IPythonView()
-        view.override_font(Pango.FontDescription(font))
-        view.set_wrap_mode(Gtk.WrapMode.CHAR)
-        sw.add(view)
-        window.add(sw)
-        window.show_all()
-
-        def _on_delete(win, _event):
-            win.hide()
-            return True
-        window.connect('delete_event', _on_delete)
-        view.updateNamespace({'gajim': app})
-        app.ipython_window = window
-
     def run(self, _application: Gtk.Application) -> None:
         # Creating plugin manager
         from gajim import plugins
@@ -677,8 +686,8 @@ class Interface:
 
         app.plugin_repository = PluginRepository()
 
-        for con in app.connections.values():
-            con.get_module('Roster').load_roster()
+        for client in app.get_clients():
+            client.get_module('Roster').load_roster()
 
         # get instances for windows/dialogs that will show_all()/hide()
         self.instances['file_transfers'] = FileTransfersWindow()
